"""LUYouTube.py"""
# -*- coding: UTF-8 -*-
__annotations__ ="""
 =======================================================
 Copyright (c) 2023-2024
 Author:
     Lisitsin Y.R.
 Project:
     LU_PY
     Python (LU)
 Module:
     LUYouTube.py

 =======================================================
"""

#------------------------------------------
# БИБЛИОТЕКИ python
#------------------------------------------
import sys

#------------------------------------------
# БИБЛИОТЕКИ сторонние
#------------------------------------------

#------------------------------------------
# БИБЛИОТЕКИ LU
#------------------------------------------
import lyrpy.LUObjectsYT as LUObjectsYT
import lyrpy.LUFile as LUFile

# --------------------------------------------
# TYouTubeObjectsItem
# --------------------------------------------
class TYouTubeObjectsItem (object):
    """TYouTubeObjectsItem"""
    luClassName = "TYouTubeObjectsItem"

    #--------------------------------------------------
    # constructor
    #--------------------------------------------------
    def __init__(self, APathDownload):
        """Constructor"""
        self.__FYouTubeObject: LUObjectsYT.TYouTubeObject = LUObjectsYT.TYouTubeObject(APathDownload)
    #endfunction

    #--------------------------------------------------
    # destructor
    #--------------------------------------------------
    def __del__(self):
        """destructor"""
        # удалить объект
        del self.__FYouTubeObject
        LClassName = self.__class__.__name__
        # s = '{} уничтожен'.format (LClassName)
        # LULog.LoggerTOOLS_AddLevel (LULog.DEBUGTEXT, s)
    #endfunction

    #--------------------------------------------------
    # @property YouTubeObject
    #--------------------------------------------------
    # getter
    @property
    def YouTubeObject(self):
    #beginfunction
        return self.__FYouTubeObject
    #endfunction
    @YouTubeObject.setter
    def YouTubeObject(self, Value: LUObjectsYT.TYouTubeObject):
    #beginfunction
        self.__FYouTubeObject = Value
    #endfunction
#endclass

# --------------------------------------------
# TYouTubeObjectsCollection
# --------------------------------------------
class TYouTubeObjectsCollection (list):
    """TObjectsCollection"""
    luClassName = "TYouTubeObjectsCollection"

    #--------------------------------------------------
    # constructor
    #--------------------------------------------------
    def __init__ (self, APathDownload: str):
        """Constructor"""
    #beginfunction
        super ().__init__ ()
        self.__FPathDownload = APathDownload
    #endfunction

    #--------------------------------------------------
    # destructor
    #--------------------------------------------------
    def __del__ (self):
        """destructor"""
    #beginfunction
        self.clear()            # удалить все items
        LClassName = self.__class__.__name__
        # s = '{} уничтожен'.format (LClassName)
        # LULog.LoggerTOOLS_AddLevel (LULog.DEBUGTEXT, s)
    #endfunction

    def AddItem (self) -> TYouTubeObjectsItem:
        """AddItem"""
    #beginfunction
        LYouTubeObjectsItem: TYouTubeObjectsItem = TYouTubeObjectsItem (APathDownload = self.__FPathDownload)
        self.append (LYouTubeObjectsItem)
        return self [self.__len__()-1]
    #endfunction

    def GetItem (self, Index: int) -> TYouTubeObjectsItem:
        """GetItem"""
    #beginfunction
        LResult: TYouTubeObjectsItem = self [Index]
        return LResult
    #endfunction

    def SetItem (self, Index: int, Value: TYouTubeObjectsItem):
        """SetItem"""
    #beginfunction
        self [Index] = Value
    #endfunction

    def FindYouTubeObjectsItemURL (self, AURL: str) -> TYouTubeObjectsItem:
        """Поиск YouTubeObjectsItem по AURL"""
    #beginfunction
        for item in self:
            LYouTubeObjectsItem:TYouTubeObjectsItem = item
            LURL = LYouTubeObjectsItem.YouTubeObject.URL
            if LURL == AURL:
                return LYouTubeObjectsItem
            #endif
        #endfor
        return None
    #endfunction
#endclass

class TYouTube(object):
    """TYouTube"""
    __annotations__ = \
        """
        TYouTube - 
        """
    luClassName = 'TYouTube'

    #--------------------------------------------------
    # constructor
    #--------------------------------------------------
    def __init__ (self, APathDownload: str):
        """ Constructor """
    #beginfunction
        super ().__init__ ()
        # YouTubeObjectsCollection
        self.__FYouTubeObjectsCollection = TYouTubeObjectsCollection (APathDownload = APathDownload)

        # self.__FAPPWorkDir = LUos.APPWorkDir ()
        # self.__FAPPWorkDir = LUos.GetCurrentDir ()
        self.__FAPPWorkDir = LUFile.ExtractFileDir(sys.argv[0])
        self.__FPathDownload = APathDownload
    #endfunction

    #--------------------------------------------------
    # destructor
    #--------------------------------------------------
    def __del__ (self):
        """ destructor """
    #beginfunction
        del self.__FYouTubeObjectsCollection
        LClassName = self.__class__.__name__
        # s = '{} уничтожен'.format (LClassName)
        # LULog.LoggerTOOLS_AddLevel (LULog.DEBUGTEXT, s)
        #print (s)
    #endfunction

    #--------------------------------------------------
    # @property YouTubeObjectsCollection
    #--------------------------------------------------
    # getter
    @property
    def YouTubeObjectsCollection (self):
    #beginfunction
        return self.__FYouTubeObjectsCollection
    #endfunction
#endclass

#---------------------------------------------------------
# main
#---------------------------------------------------------
def main ():
#beginfunction
    ...
#endfunction

#---------------------------------------------------------
#  
#---------------------------------------------------------
#beginmodule
if __name__ == "__main__":
    main()
#endif

#endmodule



"""LUStrUtils.py"""
# -*- coding: UTF-8 -*-
__annotations__ = """
 =======================================================
 Copyright (c) 2023-2024
 Author:
     Lisitsin Y.R.
 Project:
     LU_PY
     Python (LU)
 Module:
     LUStrUtils.py

 =======================================================
"""

#------------------------------------------
# БИБЛИОТЕКИ python
#------------------------------------------
import string

#------------------------------------------
# БИБЛИОТЕКИ сторонние
#------------------------------------------

#------------------------------------------
# БИБЛИОТЕКИ LU
#------------------------------------------

#---------------------------------------------------------------
#
#---------------------------------------------------------------

"""
# #import codecs
#
# f = codecs.open(filename, 'r', 'cp1251')
# u = f.read()   # now the contents have been transformed to a Unicode string
# out = codecs.open(output, 'w', 'utf-8')
# out.write(u)   # and now the contents have been output as UTF-8
"""

CRLFd = chr(13)+chr(10)
CRLFx = '\x0d'+'\x0a'
CRLF = '\r'+'\n'
"""
\n  Newline
\r  Carriage Return
\r\n    Carriage Return + Line Feed
\v or \x0b  Line Tabulation
\f or \x0c  Form Feed
\x1c    File Separator
\x1d    Group Separator
\x1e    Record Separator
\x85    Next Line (C1 Control Code)
\u2028  Unicode Line Separator
\u2029  Unicode Paragraph Separator
"""

#---------------------------------------------------------------
#
#---------------------------------------------------------------
TSysCharSet = ()
TCharSet = ()

"""
(vi) Коллекции литералов: коллекции литералов в python включают список, кортеж, словарь и наборы.
Список []:
    это список элементов, представленных в квадратных скобках с запятыми между ними.
    Эти переменные могут иметь любой тип данных и также могут быть изменены.
Кортеж ():
    это также список элементов или значений, разделенных запятыми, в круглых скобках.
    Значения могут быть любого типа данных, но не могут быть изменены.
Словарь {}:
    это неупорядоченный набор пар ключ-значение.
Set {}:
    это неупорядоченный набор элементов в фигурных скобках ‘{}’.
"""

"""
String Operators
    + Operator
    * Operator
    in Operator
Built-in String Functions    
    chr()   Converts an integer to a character
    ord()   Converts a character to an integer
    len()   Returns the length of a string
    str()   Returns a string representation of an object
String Indexing
     0  1  2  3  4  5
    -6 -5 -4 -3 -2 -1
String Slicing (Нарезка строк)
    s[2:5]
Specifying a Stride in a String Slice (Указание шага в фрагменте строки)
    [0:6:2]
Interpolating Variables Into a String (Интерполяция переменных в строку)
    f'A dog says {var}!'
Modifying Strings (Изменение строк)
    s = 'foobar'
    s[3] = 'x'              не правильно
    s = s[:3] + 'x' + s[4:] правильно
    s = s.replace('b', 'x') правильно
Built-in String Methods
    Case Conversion
        s.capitalize() 
            returns a copy of s with the first character converted to uppercase and all other characters converted to lowercase:
        s.lower() 
            returns a copy of s with all alphabetic characters converted to lowercase:
        s.swapcase() 
            returns a copy of s with uppercase alphabetic characters converted to lowercase and vice versa:
        s.title() 
            returns a copy of s in which the first letter of each word is converted to uppercase and remaining letters are lowercase:
        s.upper() 
            returns a copy of s with all alphabetic characters converted to uppercase:
    Find and Replace
        s.count(<sub>[, <start>[, <end>]]) 
            returns the number of non-overlapping occurrences of substring <sub> in s:
        s.endswith(<suffix>[, <start>[, <end>]]) 
            returns True if s ends with the specified <suffix> and False otherwise:
        s.find(<sub>[, <start>[, <end>]]) 
            You can use .find() to see if a Python string contains a particular substring. s.find(<sub>) returns the lowest index in s where substring <sub> is found:        
        s.index(<sub>[, <start>[, <end>]])
            This method is identical to .find(), except that it raises an exception if <sub> is not found rather than returning -1:
        s.rfind(<sub>[, <start>[, <end>]])
            returns the highest index in s where substring <sub> is found:
        s.rindex(<sub>[, <start>[, <end>]])
            This method is identical to .rfind(), except that it raises an exception if <sub> is not found rather than returning -1:
        s.startswith(<prefix>[, <start>[, <end>]])
            When you use the Python .startswith() method, s.startswith(<suffix>) returns True if s starts with the specified <suffix> and False otherwise:
    Character Classification
        s.isalnum() 
            returns True if s is nonempty and all its characters are alphanumeric (either a letter or a number), and False
        s.isalpha() 
            returns True if s is nonempty and all its characters are alphabetic, and False otherwise:
        s.isdigit()
            You can use the .isdigit() Python method to check if your string is made of only digits. s.isdigit() returns True if s is nonempty and all its characters are numeric digits, and False otherwise:
        s.isidentifier() 
            returns True if s is a valid Python identifier according to the language definition, and False otherwise:
        s.islower()
            returns True if s is nonempty and all the alphabetic characters it contains are lowercase, and False otherwise. Non-alphabetic characters are ignored:
        s.isprintable() 
            returns True if s is empty or all the alphabetic characters it contains are printable. It returns False if s contains at least one non-printable character. Non-alphabetic characters are ignored:
        s.isspace()
            returns True if s is nonempty and all characters are whitespace characters, and False otherwise.
        s.istitle() returns True if s is nonempty, the first alphabetic character of each word is uppercase, and all other alphabetic characters in each word are lowercase. It returns False otherwise:
        s.isupper() returns True if s is nonempty and all the alphabetic characters it contains are uppercase, and False otherwise. Non-alphabetic characters are ignored:
    String Formatting
        s.center(<width>[, <fill>])
            returns a string consisting of s centered in a field of width <width>. By default, padding consists of the ASCII space character:
        s.expandtabs(tabsize=8)
            replaces each tab character ('\t') with spaces. By default, spaces are filled in assuming a tab stop at every eighth column:
        s.ljust(<width>[, <fill>])
            returns a string consisting of s left-justified in a field of width <width>. By default, padding consists of the ASCII space character:
        s.lstrip([<chars>])
            returns a copy of s with any whitespace characters removed from the left end:
        s.replace(<old>, <new>[, <count>])
            In Python, to remove a character from a string, you can use the Python string .replace() method. s.replace(<old>, <new>) returns a copy of s with all occurrences of substring <old> replaced by <new>:
        s.rjust(<width>[, <fill>])
            returns a string consisting of s right-justified in a field of width <width>. By default, padding consists of the ASCII space character:
        s.rstrip([<chars>])
            returns a copy of s with any whitespace characters removed from the right end:
        s.strip([<chars>])
             is essentially equivalent to invoking s.lstrip() and s.rstrip() in succession. Without the <chars> argument, it removes leading and trailing whitespace:
        s.zfill(<width>)
            returns a copy of s left-padded with '0' characters to the specified <width>:
    Converting Between Strings and Lists
        s.join(<iterable>)
            returns the string that results from concatenating the objects in <iterable> separated by s.
        s.partition(<sep>)
            splits s at the first occurrence of string <sep>. The return value is a three-part tuple consisting of:
        s.rpartition(<sep>)
            functions exactly like s.partition(<sep>), except that s is split at the last occurrence of <sep> instead of the first occurrence:
        s.rsplit(sep=None, maxsplit=-1)
            Without arguments, s.rsplit() splits s into substrings delimited by any sequence of whitespace and returns the substrings as a list:
        s.split(sep=None, maxsplit=-1)
             behaves exactly like s.rsplit(), except that if <maxsplit> is specified, splits are counted from the left end of s rather than the right end:
        s.splitlines([<keepends>])
            splits s up into lines and returns them in a list. Any of the following characters or character sequences is considered to constitute a line boundary:
bytes Objects
    Defining a Literal bytes Object
        b = b'foo bar baz'
        type(b)
    The 'r' prefix may be used on a bytes literal to disable processing of escape sequences, as with strings:
        b = rb'foo\xddbar'
        >>> b
        b'foo\\xddbar'
    Defining a bytes Object With the Built-in bytes() Function
        bytes(<s>, <encoding>)
            converts string <s> to a bytes object, using str.encode() according to the specified <encoding>
        bytes(<size>)
            defines a bytes object of the specified <size>, which must be a positive integer. The resulting bytes object is initialized to null (0x00) bytes:
        bytes(<iterable>)
            defines a bytes object from the sequence of integers generated by <iterable>. <iterable> must be an iterable that generates a sequence of integers n in the range 0 ≤ n ≤ 255:
    Operations on bytes Objects
        The in and not in operators:
        The concatenation (+) and replication (*) operators:
        Indexing and slicing:
            >>> b = b'abcde'
            >>> b[2]
            99
            >>> b[1:3]
            b'bc'
        Built-in functions:
            len(b)
            min(b)
            max(b)
        bytes.fromhex(<s>)
            returns the bytes object that results from converting each pair of hexadecimal digits in <s> to the corresponding byte value. The hexadecimal digit pairs in <s> may optionally be separated by whitespace, which is ignored:
        b.hex()
            returns the result of converting bytes object b into a string of hexadecimal digit pairs. That is, it does the reverse of .fromhex():
    bytearray Objects
        There is no dedicated syntax built into Python for defining a bytearray literal, like the 'b' prefix that may be used to define a bytes object. A bytearray object is always created using the bytearray() built-in function:
            ba = bytearray('foo.bar.baz', 'UTF-8')
            bytearray(6)
                bytearray(b'\x00\x00\x00\x00\x00\x00')
            bytearray([100, 102, 104, 106, 108])
                bytearray(b'dfhjl')
        bytearray objects are mutable. You can modify the contents of a bytearray object using indexing and slicing:
                      
"""
# Алфавиты: все заглавные (A-Z) и строчные (a-z) алфавиты.

# Цифры: все цифры 0-9.
cDigitChars = tuple(string.digits)
cBrackets = ('(', ')', '[', ']', '{', '}')
# Специальные символы (пунктуация)  ” ‘ l ; : ! ~ @ # $ % ^ ` & * ( ) _ + – = { } [ ] \ .
cWordDelimitersFull = tuple(string.punctuation)
cWordDelimiters = ('|', ';')
cWordDelimiter = '|'

#---------------------------------------------------------------
# PrintableStr
#---------------------------------------------------------------
def PrintableStr(s: str) -> str:
    return ''.join(c for c in s if c.isalpha()
                   or c.isnumeric() or c.isspace()
                   or c in string.printable
                   )

#---------------------------------------------------------------
# MakeStr return a string of length N filled with character C. }
#---------------------------------------------------------------
def MakeStr (C: str, N: int) -> str:
    """MakeStr"""
#beginfunction
    LResult = ''
    if (len(C)==1) and (N > 0) and (N < 255):
        LResult = C*N
    return LResult
#endfunction

#---------------------------------------------------------------
# CharFromSet
#---------------------------------------------------------------
def CharFromSet (C: TCharSet) -> str:
    """CharFromSet"""
#beginfunction
    for i in range(0,255,1):
        if chr(i) in C:
            return chr(i)
        #endif
    #endfor
    return '?'
#endfunction

#--------------------------------------------------------------------
# AddChar
#--------------------------------------------------------------------
def AddChar (APad, AInput, ALength) -> str:
    """AddChar"""
#beginfunction
    x = len (AInput)
    for i in range (x, ALength, len (APad)):
        AInput = APad + AInput
    #endfor
    LAddChar = AInput
    return LAddChar
#endfunction

#--------------------------------------------------------------------
# AddCharR
#--------------------------------------------------------------------
def AddCharR (APad, AInput, ALength):
    """AddCharR"""
#beginfunction
    x = len (AInput)
    for i in range (x, ALength, len (APad)):
        AInput = AInput + APad
    #endfor
    LAddCharR = AInput
    return LAddCharR
#endfunction

#---------------------------------------------------------------
# Trim
#---------------------------------------------------------------
def Trim (s: str) -> str:
    """Trim"""
#beginfunction
    return s.strip()
#endfunction

#---------------------------------------------------------------
# TrimL
#---------------------------------------------------------------
def TrimL (s: str) -> str:
    """TrimL"""
#beginfunction
    return s.lstrip()
#endfunction

#---------------------------------------------------------------
# TrimR
#---------------------------------------------------------------
def TrimR (s: str) -> str:
    """TrimR"""
#beginfunction
    return s.rstrip()
#endfunction

#---------------------------------------------------------------
# TrimChar
#---------------------------------------------------------------
def TrimChar (s: str, c: str) -> str:
    """TrimChar"""
#beginfunction
    return s.strip (c)
#endfunction

#---------------------------------------------------------------
# TrimCharL
#---------------------------------------------------------------
def TrimCharL (s: str, c: str) -> str:
    """TrimCharL"""
#beginfunction
    return s.lstrip (c)
#endfunction

#---------------------------------------------------------------
# TrimCharR
#---------------------------------------------------------------
def TrimCharR (s: str, c: str) -> str:
    """TrimCharR"""
#beginfunction
    return s.rstrip (c)
#endfunction

#--------------------------------------------------------------------
# WordCount
#--------------------------------------------------------------------
def WordCount (AString: str, AWordDelims) -> int:
    """WordCount"""
#beginfunction
    LArray = AString.split (AWordDelims)
    LWordCount = len (LArray)
    return LWordCount
#endfunction

#--------------------------------------------------------------------
# ExtractWord
#--------------------------------------------------------------------
def ExtractWord (i: int, AString: str, AWordDelims):
    """ExtractWord"""
    #beginfunction
    LArray = AString.split (AWordDelims)
    if (i > 0) and (i <= len(LArray) + 1):
        try:
            LExtractWord = LArray [i-1]
        except IndexError as ERROR:
            LExtractWord = ""
            ...
    else:
        LExtractWord = ""
    #endif
    return LExtractWord
#endfunction

#--------------------------------------------------------------------
# ExistWord
#--------------------------------------------------------------------
def ExistWord (AString: str, AWordDelims, AWord: str):
    """ExistWord"""
#beginfunction
    n = WordCount (AString, AWordDelims)
    for i in range (1, n+1, 1):
        s = ExtractWord (i, AString, AWordDelims)
        if AWord.upper() == s.upper ():
            return True
        #endif
    #endfor
    return False
#endfunction

#---------------------------------------------------------------
# GetParamFromString
#---------------------------------------------------------------
def GetParamFromString (AParamName: str, AParamValues: str,
    AParamNames: (), AWordDelims: TCharSet) -> str:
    """GetParamFromString"""
#beginfunction
    LResult = ''
    i = 0
    for ParamName in AParamNames:
        i = i + 1
        if ParamName.upper() == AParamName.upper():
            LResult = Trim (ExtractWord (i, AParamValues, AWordDelims))
            return LResult
        #endif
    #endfor
    return LResult
#endfunction

#---------------------------------------------------------------
# SetParamToString
#---------------------------------------------------------------
def SetParamToString (AParamName: str, AParamValues: str,
    AParamNames: (), AWordDelims: TCharSet, AValue: str):
    """SetParamToString"""
#beginfunction
    LStroka = AParamValues
    s = ''
    i = 0
    for ParamName in AParamNames:
        i = i + 1
        if ParamName.upper() == AParamName.upper():
            s = s + AValue
        else:
            s = s + ExtractWord (i, LStroka, AWordDelims)
        #endif
        if i != len(AParamNames):
            s = s + CharFromSet (AWordDelims)
        #endif
    #endfor
    return s
#endfunction

#---------------------------------------------------------------
# DelChars
#---------------------------------------------------------------
def DelChars (s: str, c: str) -> str:
    """DelChars"""
#beginfunction
    LResult = s.replace(c, '')
    return LResult
#endfunction

#---------------------------------------------------------------
# DelSpace
#---------------------------------------------------------------
def DelSpaces (s: str) -> str:
    """DelSpaces"""
#beginfunction
    LResult = DelChars(s, ' ')
    return LResult
#endfunction

#---------------------------------------------------------------
# ReplaceChars
#---------------------------------------------------------------
def ReplaceChars (s, sOld, sNew: str) -> str:
    """ReplaceChars"""
#beginfunction
    LResult = s.replace (sOld, sNew)
    return LResult
#endfunction

#---------------------------------------------------------------
# CenterStr
#---------------------------------------------------------------
def CenterStr (s: str, c: str, ALen: int) -> str:
    """CenterStr"""
#beginfunction
    return s.center (len(s)+ALen, c)
#endfunction

#---------------------------------------------------------------
# strtobool
#---------------------------------------------------------------
def strtobool (val: str):
    """Convert a string representation of truth to true (1) or false (0).
    True values are 'y', 'yes', 't', 'true', 'on', and '1'; false values
    are 'n', 'no', 'f', 'false', 'off', and '0'.  Raises ValueError if
    'val' is anything else.
    """
#beginfunction
    if val.lower() in ('y', 'yes', 't', 'true', 'on', '1'):
        return 1
    elif val.lower() in ('n', 'no', 'f', 'false', 'off', '0'):
        return 0
    else:
        raise ValueError("invalid truth value %r" % (val,))
    #endif
#endfunction

#---------------------------------------------------------------
# booltostr
#---------------------------------------------------------------
def booltostr (val: bool) -> str:
    """booltostr"""
#beginfunction
    if val:
        return '1'
    else:
        return '0'
    #endif
#endfunction

"""
function IsWild (InputStr, Wilds: string; IgnoreCase: Boolean): Boolean;

    function SearchNext (var Wilds: string): Integer;
    { looking for next *, returns position and string until position }
    begin
        Result := Pos ('*', Wilds);
        if Result > 0 then
            Wilds := Copy (Wilds, 1, Result - 1);
    end;

var
    CWild, CInputWord: Integer; { counter for positions }
    I, LenHelpWilds: Integer;
    MaxInputWord, MaxWilds: Integer; { Length of InputStr and Wilds }
    HelpWilds: string;
begin
    if Wilds = InputStr then
    begin
        Result := True;
        Exit;
    end;
    repeat { delete '**', because '**' = '*' }
        I := Pos ('**', Wilds);
        if I > 0 then
            Wilds := Copy (Wilds, 1, I - 1) + '*' + Copy (Wilds, I + 2, MaxInt);
    until I = 0;
    if Wilds = '*' then
    begin { for fast end, if Wilds only '*' }
        Result := True;
        Exit;
    end;
    MaxInputWord := Length (InputStr);
    MaxWilds := Length (Wilds);
    if IgnoreCase then
    begin { upcase all letters }
        InputStr := AnsiUpperCase (InputStr);
        Wilds := AnsiUpperCase (Wilds);
    end;
    if (MaxWilds = 0) or (MaxInputWord = 0) then
    begin
        Result := False;
        Exit;
    end;
    CInputWord := 1;
    CWild := 1;
    Result := True;
    repeat
        if InputStr[CInputWord] = Wilds[CWild] then
        begin { equal letters }
      { goto next letter }
            Inc (CWild);
            Inc (CInputWord);
            Continue;
        end;
        if Wilds[CWild] = '?' then
        begin { equal to '?' }
      { goto next letter }
            Inc (CWild);
            Inc (CInputWord);
            Continue;
        end;
        if Wilds[CWild] = '*' then
        begin { handling of '*' }
            HelpWilds := Copy (Wilds, CWild + 1, MaxWilds);
            I := SearchNext (HelpWilds);
            LenHelpWilds := Length (HelpWilds);
            if I = 0 then
            begin
        { no '*' in the rest, compare the ends }
                if HelpWilds = '' then
                    Exit; { '*' is the last letter }
        { check the rest for equal Length and no '?' }
                for I := 0 to LenHelpWilds - 1 do
                begin
                    if (HelpWilds[LenHelpWilds - I] <> InputStr
                        [MaxInputWord - I]) and
                        (HelpWilds[LenHelpWilds - I] <> '?') then
                    begin
                        Result := False;
                        Exit;
                    end;
                end;
                Exit;
            end;
      { handle all to the next '*' }
            Inc (CWild, 1 + LenHelpWilds);
            I := FindPart (HelpWilds, Copy(InputStr, CInputWord, MaxInt));
            if I = 0 then
            begin
                Result := False;
                Exit;
            end;
            CInputWord := I + LenHelpWilds;
            Continue;
        end;
        Result := False;
        Exit;
    until (CInputWord > MaxInputWord) or (CWild > MaxWilds);
  { no completed evaluation }
    if CInputWord <= MaxInputWord then
        Result := False;
    if (CWild <= MaxWilds) and (Wilds[MaxWilds] <> '*') then
        Result := False;
end;
"""

#---------------------------------------------------------
# main
#---------------------------------------------------------
def main ():
#beginfunction
    ...
#endfunction

#---------------------------------------------------------
#
#---------------------------------------------------------
#beginmodule
if __name__ == "__main__":
    main()
#endif

#endmodule



"""LUConsole.py"""
# -*- coding: UTF-8 -*-
__annotations__ = """
 =======================================================
 Copyright (c) 2023-2025
 Author:
     Lisitsin Y.R.
 Project:
     LU_PY
     Python (LU)
 Module:
     LUConsole.py

 =======================================================
"""

#------------------------------------------
# БИБЛИОТЕКИ python
#------------------------------------------
import sys
import enum

#------------------------------------------
# БИБЛИОТЕКИ сторонние
#------------------------------------------
from colorama import Fore, Back, Style

#------------------------------------------
# БИБЛИОТЕКИ LU
#------------------------------------------
import lyrpy.LUSupport as LUSupport

#------------------------------------------
# CONST
#------------------------------------------
sBEGIN_hex = '\x1b['
sBEGIN_Unicode = '\u001b'
sBEGIN_oct = '\33['
sBEGIN = sBEGIN_hex
sEND = 'm'
sRESET = sBEGIN_oct+'0'+sEND          # сброс к начальным значениям
sRESET_1 = sBEGIN_oct+'0'+';'+sEND      # вернуться к начальному стилю
sRESET_2 = sBEGIN_oct+sEND

sR = '\r'
sB = '\b' # символ возврата

sERASE_LINE = '\x1b[2K' # erase line command
sCURSOR_UP_ONE = '\033[K'

#--------------------------------------
# Изменения стиля (Styles)
#--------------------------------------
cS_BOLD       = '01' # Жирный
cS_02         = '02' # Блеклый
cS_ITALIC     = '03' # Курсив
cS_UNDERLINE  = '04' # Подчёркнутый
cS_05         = '05' # Мигание
cS_06         = '06'
cS_REVERSE    = '07' # Реверс
cS_08         = '08'
cS_09         = '09' # Зачёркнутый
@enum.unique
class cStyles(enum.Enum):
    """cStyles"""
    BOLD       = cS_BOLD
    ITALIC     = cS_ITALIC
    UNDERLINE  = cS_UNDERLINE
    REVERSE    = cS_REVERSE
#endclass
#--------------------------------------
cStylesList = [cStyles.BOLD, cStyles.ITALIC, cStyles.UNDERLINE, cStyles.REVERSE]

#--------------------------------------
# Изменения цвета шрифта
#--------------------------------------
cFG8_BLACK      = '30' # Чёрный
cFG8_RED        = '31' # Красный
cFG8_GREEN      = '32' # Зелёный
cFG8_YELLOW     = '33' # Жёлтый
cFG8_BLUE       = '34' # Синий
cFG8_MAGENTA    = '35' # Фиолетовый (пурпурный цвет)
cFG8_CYAN       = '36' # Бирюзовый (голубой цвет)
cFG8_WHITE      = '37' # Белый
@enum.unique
class cFG8(enum.Enum):
    """cFG8"""
    BLACK   = cFG8_BLACK
    RED     = cFG8_RED
    GREEN   = cFG8_GREEN
    YELLOW  = cFG8_YELLOW
    BLUE    = cFG8_BLUE
    MAGENTA = cFG8_MAGENTA
    CYAN    = cFG8_CYAN
    WHITE   = cFG8_WHITE
#endclass
#--------------------------------------
cFG8List = [cFG8.BLACK, cFG8.RED, cFG8.GREEN, cFG8.YELLOW, cFG8.BLUE, cFG8.MAGENTA, cFG8.CYAN, cFG8.WHITE]

#--------------------------------------
# Изменения цвета фона
#--------------------------------------
cBG8_BLACK      = '40' # Чёрный
cBG8_RED        = '41' # Красный
cBG8_GREEN      = '42' # Зелёный
cBG8_YELLOW     = '43' # Жёлтый
cBG8_BLUE       = '44' # Синий
cBG8_MAGENTA    = '45' # Фиолетовый (пурпурный цвет)
cBG8_CYAN       = '46' # Бирюзовый (голубой цвет)
cBG8_WHITE      = '47' # Белый
@enum.unique
class cBG8(enum.Enum):
    """cBG8"""
    BLACK   = cBG8_BLACK
    RED     = cBG8_RED
    GREEN   = cBG8_GREEN
    YELLOW  = cBG8_YELLOW
    BLUE    = cBG8_BLUE
    MAGENTA = cBG8_MAGENTA
    CYAN    = cBG8_CYAN
    WHITE   = cBG8_WHITE
#endclass
#--------------------------------------
cBG8List = [cBG8.BLACK, cBG8.RED, cBG8.GREEN, cBG8.YELLOW, cBG8.BLUE, cBG8.MAGENTA, cBG8.CYAN, cBG8.WHITE]

#--------------------------------------
# Избранные цвета 8
#--------------------------------------
# красный текст - для обозначения ошибок
# \033[ 01; 03; 04; 07; '__'; '__' m
ERROR_ESC = '\033['+'31'+'m'
ERROR_s = ''
ERROR_cFG8 = cFG8_RED
ERROR_cBG8 = ''
# жирный красный текст - для обозначения критических ошибок
ERROR_CRITICAL_ESC = '\033['+'01'+'31'+'m'
ERROR_CRITICAL_s = cS_BOLD
ERROR_CRITICAL_cFG8 = cFG8_RED
ERROR_CRITICAL_cBG8 = ''
# зеленый текст - успешное выполнение
SUCCESS_ESC = '\033['+'32'+'m'
SUCCESS_s = ''
SUCCESS_cFG8 = cFG8_GREEN
SUCCESS_sBG8 = ''
# красный курсив - текст ошибки
ERROR_TEXT_ESC = '\033['+'03'+'31'+'m'
ERROR_TEXT_s = cS_ITALIC
ERROR_TEXT_cFG8 = cFG8_RED
ERROR_TEXT_cBG8 = ''
# выделение основного, как будто жёлтым маркером
MARKER_ESC = '\033['+'43'+'m'
MARKER_s = ''
MARKER_cFG8 = ''
MARKER_cBG8 = cBG8_YELLOW
# жирный белый на черном
BOLD_ESC = '\033['+'01'+'31'+'40'+'m'
BOLD_s = cS_BOLD
BOLD_cFG8 = cFG8_WHITE
BOLD_cBG8 = cBG8_BLACK

#--------------------------------------
# Больше цветов: аж целых 256
# Совсем много цветов
# Этот формат не всегда поддерживается стандартными консолями.
#--------------------------------------
# Некоторые будут негодовать: "256 цветов и нет моего любимого терракотового, какой ужас!".
# Для таких ценителей существует формат, который уже поддерживает 24 битные цвета (3 канала RGB по 256 градаций).
# Для не ценителей поясню, что терракотовый кодируется как — (201, 100, 59) или #c9643b.
# Синтаксис в этом формате выглядит вот так:
# цвет текста
# \33[+38;2;⟨r⟩;⟨g⟩;⟨b⟩ m
# цвет фона
# \33[+48;2;⟨r⟩;⟨g⟩;⟨b⟩ m
#--------------------------------------
# \033[ 01; 03; 04; 07; 38;05;222; 48;05;22 m
#--------------------------------------
# Изменения цвета шрифта
#--------------------------------------
sFG256_BEGIN = '38;05;'
#--------------------------------------
# Изменения цвета фона
#--------------------------------------
sBG256_BEGIN = '48;05;'
#--------------------------------------
#--------------------------------------
# Избранные цвета 256
#--------------------------------------
sFG256_01 = '38;05;'+'15'
sBG256_01 = '48;05;'+'21'
sColor256_01 = sBEGIN_oct+cS_BOLD+';'+sFG256_01+';'+sBG256_01+sEND

#--------------------------------------
# colorama
#--------------------------------------
# список доступных фронтальных цветов
#--------------------------------------
FOREGROUND = [Fore.BLACK, Fore.RED, Fore.GREEN, Fore.YELLOW, Fore.BLUE, Fore.MAGENTA, Fore.CYAN, Fore.WHITE]
#--------------------------------------
# список доступных фоновых цветов
#--------------------------------------
BACKGROUND = [Back.BLACK, Back.RED, Back.GREEN, Back.YELLOW, Back.BLUE, Back.MAGENTA, Back.CYAN, Back.WHITE]
#
BRIGHTNESS = [Style.DIM, Style.NORMAL, Style.BRIGHT]

#-------------------------------------------------
# FormatColorStr (s, AStyles:()='', AFG8:str='', ABG8:str='', AFG256:str='', ABG256:str='', AESC:str=''):
#-------------------------------------------------
def FormatColorStr (s, **kwargs) -> str:
    """FormatColorStr"""
#beginfunction
    AStyles:() = kwargs.get('AStyles')
    AFG8:str = kwargs.get ('AFG8')
    ABG8:str = kwargs.get ('ABG8')
    AFG256:str = kwargs.get ('AFG256')
    ABG256:str = kwargs.get ('ABG256')
    AESC:str = kwargs.get ('AESC')

    LResult = ''
    if  AESC is not None:
        LResult = LResult + AESC + s + sRESET
    else:
        LStyles = LUSupport.TupleToStr (AStyles)
        # --------------------------------------------
        if len(LStyles) > 0 \
                or AFG8 is not None or ABG8 is not None \
                or AFG256 is not None or ABG256 is not None:
            LResult = sBEGIN
        # --------------------------------------------
        if len (LStyles) > 0:
            LResult = LResult + LStyles
        # --------------------------------------------
        if AFG8 is not None:
            if len (LStyles) > 0:
                LResult = LResult + ';' + AFG8
            else:
                LResult = LResult + AFG8
        # --------------------------------------------
        if ABG8 is not None:
            if len (LStyles) > 0 or AFG8 is not None:
                LResult = LResult + ';' + ABG8
            else:
                LResult = LResult + ABG8
        # --------------------------------------------
        if AFG8 is None and ABG8 is None:
            if AFG256 is not None:
                if len (LStyles) > 0:
                    LResult = LResult + ';' + sFG256_BEGIN + AFG256
                else:
                    LResult = LResult + sFG256_BEGIN +AFG256
            # --------------------------------------------
            if ABG256 is not None:
                if len (LStyles) > 0 or AFG256 is not None:
                    LResult = LResult + ';' + sBG256_BEGIN + ABG256
                else:
                    LResult = LResult + sBG256_BEGIN + ABG256
        # --------------------------------------------
        if len (LResult) > 0:
            LResult = LResult + sEND + s + sRESET
        else:
            LResult = s
    return LResult
#endfunction

#-------------------------------------------------
# Write (s, AStyles:()='', AFG8:str='', ABG8:str='', AFG256:str='', ABG256:str='', AESC:str=''):
#-------------------------------------------------
def Write (s, **kwargs):
    """Write"""
#beginfunction
    _s = s
    if LUSupport.IsTerminal():
        sys.stdout.write (_s)
    else:
        if len(kwargs):
            __s = FormatColorStr(_s, **kwargs)
            sys.stdout.write (__s)
        else:
            sys.stdout.write (_s)
        #endif
    #endif
    sys.stdout.flush ()
#endfunction

#-------------------------------------------------
# WriteLN (s, AStyles:()='', AFG8:str='', ABG8:str='', AFG256:str='', ABG256:str='', AESC:str=''):
#-------------------------------------------------
def WriteLN (s, **kwargs):
    """WriteLN"""
#beginfunction
    Write (s, **kwargs)
    sys.stdout.write ('\n')
    sys.stdout.flush ()
#endfunction

#-------------------------------------------------
# ClearLine
#-------------------------------------------------
def ClearLine():
    """ClearLine"""
#beginfunction
    # if ISTerminal():
    #     sys.stdout.write ('\r')
    # else:
    #     sys.stdout.write(sCURSOR_UP_ONE)
    #     sys.stdout.write(sERASE_LINE+'\r')
    # #endif
    sys.stdout.write ('\r')
#endfunction

#-------------------------------------------------
# ReadParam
#-------------------------------------------------
def ReadParam (ATitle: str, ADefault: str) -> str:
    """ReadParam"""
#beginfunction
    WriteLN ('Введите ('+ATitle+')['+ADefault+']: ', AStyles=cS_ITALIC, AFG8=cFG8_RED, ABG8=cBG8_WHITE)
    LReadParam: str = input ("")
    if LReadParam == "":
       LReadParam = ADefault
    #endif
    return LReadParam
#endfunction

#--------------------------------------------------------------------
# Pause
#--------------------------------------------------------------------
def Pause (Aprompt: str = ''):
    """Pause"""
#beginfunction
    if Aprompt == '':
        Aprompt = "Press any key to continue"
    #endif
    if Aprompt != '':
        WriteLN (Aprompt, AStyles=cS_ITALIC, AFG8=cFG8_RED, ABG8=cBG8_WHITE)
    #endif
    x = sys.stdin.read (1)
#endfunction

#--------------------------------------------------------------------
# pause2
#--------------------------------------------------------------------
def pause2 (ADelay:int=0, Aprompt:str=''):
    """pause2"""
#beginfunction
    LDelay = ADelay
    LPrompt = Aprompt
    LLoop = 0
    LCounter = 0
    LInterval = 0.2
    LPause = -1
    if Aprompt == '':
        LPrompt = "Press any key to continue"
    #endif
    if LPrompt != '':
        WriteLN(LPrompt)
    #endif
    if LDelay > 0:
        LDelay = LDelay + 1
        while LPause == -1 and LDelay > 1.0 + LInterval:
            LDelay = LDelay - LInterval
            #LCounter = "[" + int(LDelay) + "]:"
            #Write (LCounter)
            #sleep (LInterval)
            #for loop in range (1, len(LCounter), 1):
            #    Write (chr(8)+" "+chr(8))
            #endfor
            #if kbhit():
                #LPause = sys.stdin.read(1)
            #endif
        #endwhile
    else:
        LPause = sys.stdin.read(1)
    #endif
#endfunction

#------------------------------------------------------
# PasswdFromKbd
#------------------------------------------------------
#def PasswdFromKbd(Prompt = ''):
##beginfunction
#    WriteLN ("w/n",  "")
#    WriteLN ("w+/n", "$Admin_Password_Desc")
#    PasswdFromKbd = fnGetM("*")
#endfunction

#------------------------------------------------------
# main
#------------------------------------------------------
def main ():
#beginfunction
    print('main LUConsole.py ...')
#endfunction

#------------------------------------------
#
#------------------------------------------
#beginmodule
if __name__ == "__main__":
    main()
#endif

#endmodule



"""LUFileUtils.py"""
# -*- coding: UTF-8 -*-
__annotations__ = """
 =======================================================
 Copyright (c) 2023-2025
 Author:
     Lisitsin Y.R.
 Project:
     LU_PY
     Python (LU)
 Module:
     LUFileUtils.py

 =======================================================
"""

#------------------------------------------
# БИБЛИОТЕКИ python
#------------------------------------------
import os
import sys
import logging

#------------------------------------------
# БИБЛИОТЕКИ сторонние
#------------------------------------------

#------------------------------------------
# БИБЛИОТЕКА LU
#------------------------------------------
import lyrpy.LUFile as LUFile
import lyrpy.LULog as LULog
import lyrpy.LUDateTime as LUDateTime

#------------------------------------------
#CONST
#------------------------------------------
GLevel = 0
GFileCount = 0
GFileSize = 0
GDir = ''
GMask = '.*'
GDirCount = 0
GLevelMAX = sys.maxsize

#-------------------------------------------------------------------------------
# __OUTFILE
#-------------------------------------------------------------------------------
def __OUTFILE (s: str, OutFile: str):
#beginfunction
    if OutFile and s != '':
        if OutFile.upper () == 'CONSOLE':
            print (s)
        else:
            LUFile.WriteStrToFile (OutFile, s + '\n')
        #endif
    #endif
#endfunction

#-------------------------------------------------------------------------------
# __ListFile
#-------------------------------------------------------------------------------
def __ListFile (APathSource, AMask, APathDest,
                _OutFile, _Option, _FuncDir, _FuncFile) -> int:
    global GFileCount
    global GFileSize
#beginfunction
    LFileCount = 0
    with os.scandir(APathSource) as LFiles:
        for LFile in LFiles:
            if not LFile.is_symlink ():
                if LFile.is_file() and LUFile.CheckFileNameMask (LFile.name, AMask):
                    #------------------------------------------------------------
                    # class os.DirEntry - Это файл
                    #------------------------------------------------------------
                    LBaseName = os.path.basename (LFile.path)
                    LFileTimeSource = LUFile.GetFileDateTime (LFile.path)[2]
                    LFileSizeSource = LUFile.GetFileSize (LFile.path)

                    GFileCount = GFileCount + 1
                    LFileCount = LFileCount + 1
                    GFileSize = GFileSize + LFileSizeSource

                    match _Option:
                        case 1 | 11:
                            s = f'{LFileTimeSource:%d.%m.%Y  %H:%M} {LFileSizeSource:-17,d} {LBaseName:s}'
                        case 2 | 12:
                            s = f'{LFileTimeSource:%d.%m.%Y  %H:%M} {LFileSizeSource:-17,d} {LBaseName:s}'
                        case _:
                            s = ''
                    #endmatch
                    __OUTFILE (s, _OutFile)

                    if _FuncFile:
                        # s = f'_FuncFile: {_FuncFile.__name__:s}'
                        # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
                        _FuncFile (LUFile.ExpandFileName (LFile.path), APathDest)
                    #endif
                #endif
            #endif
            if LFile.is_dir ():  # and (not LFile.name.startswith('.')):
                #------------------------------------------------------------
                # class os.DirEntry - Это каталог
                #------------------------------------------------------------
                LBaseName = os.path.basename (LFile.path)
                LPathTimeSource = LUFile.GetDirDateTime (LFile.path) [2]

                match _Option:
                    case 1 | 11:
                        s = f'{LPathTimeSource:%d.%m.%Y  %H:%M} {'   <DIR>':17s} {LBaseName:s}'
                    case 2 | 12:
                        s = f'{LPathTimeSource:%d.%m.%Y  %H:%M} {'   <DIR>':17s} {LBaseName:s}'
                    case _:
                        s = ''
                #endmatch
                __OUTFILE (s, _OutFile)

                #------------------------------------------------------------
                #
                #------------------------------------------------------------
                if _FuncDir:
                    # s = f'_FuncDir: {_FuncDir.__name__:s}'
                    # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
                    # _FuncDir (LUFile.ExpandFileName (LFile.path), APathDest)
                    pass
                #endif
            #endif
        #endfor
    #endwith
    return LFileCount
#endfunction

#-------------------------------------------------------------------------------
# __ListDir
#-------------------------------------------------------------------------------
def __ListDir (APathSource, AMask, ASubdir, APathDest,
               _OutFile, _Option, _FuncDir, _FuncFile):
#beginfunction
    global GLevel
    global GFileCount
    global GFileSize

    #------------------------------------------------------------
    # Dir
    #------------------------------------------------------------
    LBaseName = os.path.basename (APathSource)
    LPathTimeSource = LUFile.GetDirDateTime (APathSource)[2]

    GFileCount = 0
    GFileSize = 0

    #------------------------------------------------------------
    # список файлов в каталоге
    #------------------------------------------------------------
    if _Option != 0:
        s = f"\nСодержимое папки {APathSource:s}\n"
        __OUTFILE (s, _OutFile)
    #endif
    match _Option:
        case 1 | 11:
            s = f'{LPathTimeSource:%d.%m.%Y  %H:%M} {'   <DIR>':17s} {LBaseName:s}'
        case 2 | 12:
            s = f'{LPathTimeSource:%d.%m.%Y  %H:%M} {'   <DIR>':17s} {LBaseName:s}'
        case _:
            s = ''
    #endmatch
    __OUTFILE (s, _OutFile)

    # LFileCount = __ListFile (APathSource, AMask, APathDest, _OutFile, _Option, _FuncDir, _FuncFile)
    LFileCount = __ListFile (APathSource, AMask, APathDest, _OutFile, _Option, None, _FuncFile)

    match _Option:
        case 1 | 11:
            s = f'{GFileCount:16d} файлов {GFileSize:16,d} байт'
        case 2 | 12:
            s = f'{GFileCount:16d} файлов {GFileSize:16,d} байт'
        case _:
            s = ''
    #endmatch
    __OUTFILE (s, _OutFile)
    #------------------------------------------------------------

    with os.scandir(APathSource) as LFiles:
        for LFile in LFiles:
            if not LFile.is_symlink():
                if LFile.is_dir (): # and (not LFile.name.startswith('.')):
                    #------------------------------------------------------------
                    #
                    #------------------------------------------------------------
                    if _FuncDir:
                        # s = f'_FuncDir: {_FuncDir.__name__:s}'
                        # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
                        _FuncDir (LUFile.ExpandFileName (LFile.path), APathDest)
                    #endif

                    #------------------------------------------------------------
                    # class os.DirEntry - Это каталог
                    #------------------------------------------------------------
                    # LBaseName = os.path.basename (LFile.path)
                    # LPathTimeSource = LUFile.GetFileDateTime (LFile.path)[2]

                    #------------------------------------------------------------
                    # на следующий уровень
                    #------------------------------------------------------------
                    if ASubdir:
                        GLevel = GLevel + 1
                        if APathDest != '':
                            LPathDest = os.path.join (APathDest, LFile.name)
                        else:
                            LPathDest = ''
                        #endif
                        __ListDir (LFile.path, AMask, ASubdir, LPathDest, _OutFile, _Option, _FuncDir, _FuncFile)
                    #endif
                #endif
            #endif
        #endfor
        GLevel = GLevel - 1
    #endwith
#endfunction

#-------------------------------------------------------------------------------
# BacFiles
#-------------------------------------------------------------------------------
def BacFiles (APathSource, AMask, ASubDir, APathDest,
              _OutFile, _Option, _ASync: bool=False):

    #-------------------------------------------------------------------------------
    # FuncDir
    #-------------------------------------------------------------------------------
    def FuncDir (_APathSource: str, _APathDest: str):
    #beginfunction
        LPathSource = _APathSource
        LBaseName = os.path.basename (_APathSource)
        LPathDest = os.path.join (_APathDest, LBaseName)
        # LPathTimeSource = LUFile.GetFileDateTime (_APathSource) [2]
        Lattr = LUFile.GetFileAttr (_APathSource)
        if not _ASync:
            if not LUFile.DirectoryExists(LPathDest):
                s = f'Create {LPathDest:s} ...'
                LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
                LUFile.ForceDirectories(LPathDest)
                LUFile.SetFileAttr(LPathDest, Lattr, False)
            #endif
        else:
            if not LUFile.DirectoryExists(LPathDest):
                s = f'Delete {LPathSource:s} ...'
                LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
                LUFile.DeleteDirectoryTree(LPathSource)
            #endif
        #endif
    #endfunction

    #-------------------------------------------------------------------------------
    # FuncFile 
    #-------------------------------------------------------------------------------
    def FuncFile (AFileName: str, _APathDest: str):
    #beginfunction
        LFileNameSource = AFileName
        LBaseName = os.path.basename (AFileName)
        # LFileTimeSource = LUFile.GetFileDateTime (AFileName) [2]
        LFileSizeSource = LUFile.GetFileSize (AFileName)
        LFileAttrSource = LUFile.GetFileAttr (AFileName)

        LFileNameDest = os.path.join (_APathDest, LBaseName)

        #--------------------------------------------------------------------
        LResult = LUFile.COMPAREFILETIMES(LFileNameSource, LFileNameDest)
        #--------------------------------------------------------------------
        # Check Result
        #--------------------------------------------------------------------
        LCopy = False
        LDelete = False

        match LResult:
            case -3:
                # -3 File2 could not be opened (see @ERROR for more information).
                # LFileSizeDest = 0
                # LFileTimeDest = 0
                LDelete = False
                LCopy = True
                if _ASync:
                    LCopy = False
                    LDelete = True
                #endif
            case -2:
                # -2 File1 could not be opened (see @ERROR for more information).
                LDelete = False
                LCopy = False
            case -1:
                # -1 File1 is older than file2.
                LDelete = False
                LCopy = False
            case 0:
                # 0  File1 and file2 have the same date and time.
                LDelete = False
                LCopy = False
                # LFileTimeDest = LUFile.GetFileDateTime (LFileNameDest) [2]
                LFileSizeDest = LUFile.GetFileSize (LFileNameDest)
                # LFileAttrDest = LUFile.GetFileAttr (LFileNameDest)
                if LFileSizeSource != LFileSizeDest:
                    LCopy = True
                else:
                    LUFile.SetFileAttr(LFileNameDest, LFileAttrSource, False)
                    # shutil.copystat (LFileNameSource, LFileNameDest)
                #endif
            case 1:
                # 1  File1 is more recent than file2.
                LDelete = False
                LCopy = True
        #endmatch

        #--------------------------------------------------------------------
        # Copy
        #--------------------------------------------------------------------
        if LCopy == True:
            s = f'Copy {LFileNameSource:s} -> {LFileNameDest:s} ...'
            LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
            LUFile.FileCopy (LFileNameSource, LFileNameDest, True)
        #endif

        #--------------------------------------------------------------------
        # Delete
        #--------------------------------------------------------------------
        if LDelete == True:
            s = f'Delete {LFileNameSource:s} ...'
            LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
            LUFile.FileDelete (LFileNameSource)
        #endif
    #endfunction

#beginfunction
    if (APathSource != "") and (APathDest != ""):
        # LBaseName = os.path.basename (APathSource)
        LPathDest = APathDest
        Ls = f'BacFiles: {APathSource:s} {AMask:s} => {APathDest:s} ...'
        LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, Ls)
        __ListDir (APathSource, AMask, ASubDir, LPathDest, _OutFile, _Option, FuncDir, FuncFile)
    #endif
#endfunction

#-------------------------------------------------------------------------------
# SyncFiles
#-------------------------------------------------------------------------------
def SyncFiles (APathSource, AMask, APathDest, _OutFile, _Option):
#beginfunction
    s = f'SyncFiles: {APathSource:s} {AMask:s} => {APathDest:s} ...'
    LULog.LoggerAdd (LULog.LoggerTOOLS, LULog.TEXT, s)

    BacFiles (APathSource, AMask, True, APathDest, _OutFile, _Option, False)
    BacFiles (APathDest, AMask, True, APathSource, _OutFile, _Option, True)
#endfunction

#-------------------------------------------------------------------------------
# DirFiles
#-------------------------------------------------------------------------------
def DirFiles (APathSource, AMask, ASubDir,
              _OutFile, _Option, _FuncDir, _FuncFile):
#beginfunction
    if APathSource != "":
        s = f'DirFiles: {APathSource:s} {AMask:s} ...'
        LULog.LoggerAdd (LULog.LoggerTOOLS, LULog.TEXT, s)
        __ListDir(APathSource, AMask, ASubDir, '', _OutFile, _Option, _FuncDir, _FuncFile)
    #endif
#endfunction

#-------------------------------------------------------------------------------
# __FakeFile
#-------------------------------------------------------------------------------
def __FakeFile (APathSource,
                _OutFile, _Option, _FuncDir, _FuncFile):
    global GLevel
#beginfunction
    for LFileCount in range(0, 2):
        s = f'FakeFile_{str(GLevel+1):s}_{str(LFileCount+1):s}.txt'
        LFileName = os.path.join (APathSource, s)

        LHahdle = LUFile.OpenTextFile(LFileName, '')
        LUFile.WriteTextFile(LHahdle, 'test')
        LUFile.WriteTextFile(LHahdle, 'тест')
        LUFile.CloseTextFile(LHahdle)

        if _FuncFile:
            # s = f'_FuncFile: {_FuncFile.__name__:s}'
            # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
            _FuncFile (LFileName)
        #endif
    #endfor
#endfunction

#-------------------------------------------------------------------------------
# __FakeDir
#-------------------------------------------------------------------------------
def __FakeDir (APathSource,
               _OutFile, _Option, _FuncDir, _FuncFile):
#beginfunction
    global GLevel

    #------------------------------------------------------------
    # Dir
    #------------------------------------------------------------
    # LBaseName = os.path.basename (APathSource)
    s = LUFile.ExpandFileName (APathSource)
    LULog.LoggerAdd (LULog.LoggerTOOLS, LULog.TEXT, s)

    #------------------------------------------------------------
    #
    #------------------------------------------------------------
    if _FuncDir:
        # s = f'_FuncDir: {_FuncDir.__name__:s}'
        # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
        _FuncDir (LUFile.ExpandFileName (APathSource))
    #endif

    __FakeFile (APathSource, _OutFile, _Option, _FuncDir, _FuncFile)

    #------------------------------------------------------------
    # на следующий уровень
    #------------------------------------------------------------
    if GLevel < 3:
        for LDirCount in range (0, 2):
            s = f'FakeDir_{str(GLevel+1):s}_{str(LDirCount+1):s}'
            LPathSource = os.path.join (APathSource, s)
            LUFile.ForceDirectories(LPathSource)

            GLevel = GLevel + 1
            __FakeDir(LPathSource, _OutFile, _Option, _FuncDir, _FuncFile)
        #endfor
    #endif
    GLevel = GLevel - 1
#endfunction

#-------------------------------------------------------------------------------
# FakeFiles
#-------------------------------------------------------------------------------
def FakeFiles (APathSource,
              _OutFile, _Option, _FuncDir, _FuncFile):
#beginfunction
    if APathSource != "":
        s = f'FakeFiles: {APathSource:s} ...'
        LULog.LoggerAdd (LULog.LoggerTOOLS, LULog.TEXT, s)
        __FakeDir(APathSource, _OutFile, _Option, _FuncDir, _FuncFile)
    #endif
#endfunction

#-------------------------------------------------------------------------------
# DelFiles
#-------------------------------------------------------------------------------
def DelFiles (APathSource, AMask, ASubDir, _OutFile, _Option, _Older: int):
#beginfunction

    #-------------------------------------------------------------------------------
    # DelFile
    #-------------------------------------------------------------------------------
    def DelFile (AFileName: str):
    #beginfunction
        LDay = LUDateTime.Now ()
        # print(LUFile.GetFileDateTime (AFileName))
        LFileTimeSource = LUFile.GetFileDateTime (AFileName) [2]
        # print ((LDay - LFileTimeSource).days)
        if (LDay - LFileTimeSource).days > _Older:
            s = f'Delete {AFileName:s} ...'
            LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
            LUFile.FileDelete(AFileName)
        #endif
    #endfunction

#beginfunction
    if APathSource != "":
        s = f'DelFiles: {APathSource:s} {AMask:s} ...'
        LULog.LoggerAdd (LULog.LoggerTOOLS, LULog.TEXT, s)
        __ListDir (APathSource, AMask, ASubDir, '', _OutFile, _Option, None, DelFile)
    #endif
#endfunction

#------------------------------------------
def main ():
#beginfunction
    print('main LUFileUtils.py ...')
#endfunction

#------------------------------------------
#
#------------------------------------------
#beginmodule
if __name__ == "__main__":
    main()
#endif

#endmodule



"""LUVersion.py"""
# -*- coding: UTF-8 -*-
__annotations__ = """
 =======================================================
 Copyright (c) 2023-2024
 Author:
     Lisitsin Y.R.
 Project:
     LU_PY
     Python (LU)
 Module:
     LUVersion.py

 =======================================================
"""
#------------------------------------------
# БИБЛИОТЕКИ python
#------------------------------------------
import os

#------------------------------------------
# БИБЛИОТЕКИ сторонние
#------------------------------------------
import datetime

# import win32api

# if platform.system() == 'Windows':
#     import win32api
#     import win32con
# #endif

#------------------------------------------
# БИБЛИОТЕКА LU
#------------------------------------------

class TVersionInfo:
    """TVersionInfo"""
    luClassName = "TVersionInfo"
    
    #--------------------------------------------------
    # constructor
    #--------------------------------------------------
    def __init__(self):
        self.__FFileName = ''
        self.__FInfo = None
        self.__FFileDate = 0
        self.__FFileVersion = ''
        self.__FFileInfoSize = 0
        self.__FInfoSize = 0
        self.__FFileInfo = None
        self.__FCompanyName = ''
        self.__FFileDescription = ''
        self.__FInternalName = ''
        self.__FLegalTrademarks = ''
        self.__FLegalCopyright = ''
        self.__FProductName = ''
        self.__FOriginalFilename = ''
        self.__FProductVersion = ''
        self.__FComments = ''

        self.__lang = ''
        self.__codepage = ''

        self.__FTmp = None
        self.__FTransInfo = None
        self.__FTransInfoSize = 0

    #--------------------------------------------------
    # destructor
    #--------------------------------------------------
    def __del__(self):
    #beginfunction
        LClassName = self.__class__.__name__
        # s = '{} уничтожен'.format (LClassName)
        #print (s)
    #endfunction

    #--------------------------------------------------
    # @property FileName
    #--------------------------------------------------
    # getter
    @property
    def FileName(self):
    #beginfunction
        return self.__FFileName
    #endfunction
    # setter
    @FileName.setter
    def FileName(self, Value: str):
        propNames = ('Comments', 'InternalName', 'ProductName',
                     'CompanyName', 'LegalCopyright', 'ProductVersion',
                     'FileDescription', 'LegalTrademarks', 'PrivateBuild',
                     'FileVersion', 'OriginalFilename', 'SpecialBuild')

        props = {'FixedFileInfo': None, 'StringFileInfo': None, 'FileVersion': None}
    #beginfunction
        if Value == '':
            return
        self.__FFileName = Value
        # Get the size of the FileVersionInformatioin
        if self.__FInfoSize > 0:
            pass
        #self.__FInfoSize = win32api.GetFileVersionInfoSize(self.__FFileName.encode(), None)
        # If InfoSize = 0, then the file may not exist, or
        # it may not have file version information in it.
        #if self.__FInfoSize == 0:
        #    raise Exception.Create("Can''t get file version information for "+self.__FFileName)

        #file modification
        #self.__FFileDate = FileDateToDateTime(FileAge(Value))
        LFileTimeSource = os.path.getmtime(self.__FFileName)
        #convert timestamp into DateTime object
        self.__FFileDate = datetime.datetime.fromtimestamp(LFileTimeSource)

        # Get the information
        #self.__FInfo = win32api.GetFileVersionInfo(self.__FFileName, 0, self.__FInfoSize, self.__FInfo)
        # backslash as parm returns dictionary of numeric info corresponding to VS_FIXEDFILEINFO struc
        self.__FInfo = win32api.GetFileVersionInfo (self.__FFileName, '\\')
        props ['FixedFileInfo'] = self.__FInfo
        props ['FileVersion'] = "%d.%d.%d.%d" % (self.__FInfo ['FileVersionMS'] / 65536,
                                                 self.__FInfo ['FileVersionMS'] % 65536,
                                                 self.__FInfo ['FileVersionLS'] / 65536,
                                                 self.__FInfo ['FileVersionLS'] % 65536)
        self.__FFileVersion = props ['FileVersion']

        # \VarFileInfo\Translation returns list of available (language, codepage)
        # pairs that can be used to retreive string info. We are using only the first pair.
        self.__lang, self.__codepage = win32api.GetFileVersionInfo (self.__FFileName, '\\VarFileInfo\\Translation') [0]
        #print ('__lang = ',self.__lang)
        #print ('__codepage = ', self.__codepage)

        # any other must be of the form \StringfileInfo\%04X%04X\parm_name, middle
        # two are language/codepage pair returned from above
        strInfo = {}
        for propName in propNames:
            strInfoPath = u'\\StringFileInfo\\%04X%04X\\%s' % (self.__lang, self.__codepage, propName)
            #print str_info
            strInfo [propName] = win32api.GetFileVersionInfo (self.__FFileName, strInfoPath)
        props ['StringFileInfo'] = strInfo
        self.__FCompanyName = strInfo ['CompanyName']
        self.__FFileDescription = strInfo ['FileDescription']
        self.__FInternalName = strInfo ['InternalName']
        self.__FLegalCopyright = strInfo ['LegalCopyright']
        self.__FLegalTrademarks = strInfo ['LegalTrademarks']
        self.__FOriginalFilename = strInfo ['OriginalFilename']
        self.__FProductName = strInfo ['ProductName']
        self.__FProductVersion = strInfo ['ProductVersion']
        self.__FComments = strInfo ['Comments']
    #endfunction

    #--------------------------------------------------
    # @property Major1
    #--------------------------------------------------
    # getter
    @property
    def Major1(self):
    #beginfunction
        LResult = int(self.__FInfo ['FileVersionMS'] / 65536)
        return LResult
    #endfunction

    #--------------------------------------------------
    # @property Major2
    #--------------------------------------------------
    # getter
    @property
    def Major2(self):
    #beginfunction
        LResult = int(self.__FInfo ['FileVersionMS'] % 65536)
        return LResult
    #endfunction

    #--------------------------------------------------
    # @property Minor1
    #--------------------------------------------------
    # getter
    @property
    def Minor1(self):
    #beginfunction
        LResult = int(self.__FInfo ['FileVersionLS'] / 65536)
        return LResult
    #endfunction

    #--------------------------------------------------
    # @property Minor2
    #--------------------------------------------------
    # getter
    @property
    def Minor2(self):
    #beginfunction
        LResult = int(self.__FInfo ['FileVersionLS'] % 65536)
        return LResult
    #endfunction

    #--------------------------------------------------
    # @property Lang1
    #--------------------------------------------------
    # getter
    @property
    def Lang1(self):
    #beginfunction
        #Result = self.__FTransInfo.dwLang1
        LResult = ''
        return LResult
    #endfunction

    #--------------------------------------------------
    # @property Lang2
    #--------------------------------------------------
    # getter
    @property
    def Lang2(self):
    #beginfunction
        #Result = self.__FTransInfo.dwLang2
        LResult = ''
        return LResult
    #endfunction

    #--------------------------------------------------
    # @property LangCharSet
    #--------------------------------------------------
    # getter
    @property
    def LangCharSet(self):
    #beginfunction
        #Result = IntToHex(Lang1,4)+IntToHex(Lang2,4)
        LResult = ''
        return LResult
    #endfunction

    #--------------------------------------------------
    # @property FileVersion
    #--------------------------------------------------
    # getter
    @property
    def FileVersion(self):
    #beginfunction
        LResult = self.__FFileVersion
        return LResult
    #endfunction

    #--------------------------------------------------
    # @property FileDate
    #--------------------------------------------------
    # getter
    @property
    def FileDate(self):
    #beginfunction
        LResult = self.__FFileDate
        return LResult
    #endfunction

    #--------------------------------------------------
    # @property CompanyName
    #--------------------------------------------------
    # getter
    @property
    def CompanyName(self):
    #beginfunction
        LResult = self.__FCompanyName
        return LResult
    #endfunction

    #--------------------------------------------------
    # @property FileDescription
    #--------------------------------------------------
    # getter
    @property
    def FileDescription(self):
    #beginfunction
        LResult = self.__FFileDescription
        return LResult
    #endfunction

    #--------------------------------------------------
    # @property InternalName
    #--------------------------------------------------
    # getter
    @property
    def InternalName(self):
    #beginfunction
        LResult = self.__FInternalName
        return LResult
    #endfunction

    #--------------------------------------------------
    # @property LegalCopyright
    #--------------------------------------------------
    # getter
    @property
    def LegalCopyright(self):
    #beginfunction
        LResult = self.__FLegalCopyright
        return LResult
    #endfunction

    #--------------------------------------------------
    # @property LegalTrademarks
    #--------------------------------------------------
    # getter
    @property
    def LegalTrademarks(self):
    #beginfunction
        LResult = self.__FLegalTrademarks
        return LResult
    #endfunction

    #--------------------------------------------------
    # @property OriginalFilename
    #--------------------------------------------------
    # getter
    @property
    def OriginalFilename(self):
    #beginfunction
        LResult = self.__FOriginalFilename
        return LResult
    #endfunction

    #--------------------------------------------------
    # @property ProductName
    #--------------------------------------------------
    # getter
    @property
    def ProductName(self):
    #beginfunction
        LResult = self.__FProductName
        return LResult
    #endfunction

    #--------------------------------------------------
    # @property ProductVersion
    #--------------------------------------------------
    # getter
    @property
    def ProductVersion(self):
    #beginfunction
        LResult = self.__FProductVersion
        return LResult
    #endfunction

    #--------------------------------------------------
    # @property Comments
    #--------------------------------------------------
    # getter
    @property
    def Comments(self):
    #beginfunction
        LResult = self.__FComments
        return LResult
    #endfunction
#endclass

#-------------------------------------------------------------------------------
# CreateVersion
#-------------------------------------------------------------------------------
def CreateVersion (AFileName: str) -> TVersionInfo:
    """CreateVersion"""
#beginfunction
    LResult:TVersionInfo = TVersionInfo ()
    LResult.__FFileName = AFileName
    return LResult
#endfunction

#---------------------------------------------------------
# main
#---------------------------------------------------------
def main ():
#beginfunction
    ...
#endfunction

#---------------------------------------------------------
#
#---------------------------------------------------------
#beginmodule
if __name__ == "__main__":
    main()
#endif

#endmodule



"""LUParserARG.py"""
# -*- coding: UTF-8 -*-
__annotations__ = """
 =======================================================
 Copyright (c) 2023-2025
 Author:
     Lisitsin Y.R.
 Project:
     LU_PY
     Python (LU)
 Module:
     LUParserARG.py

 =======================================================
"""

#------------------------------------------
# БИБЛИОТЕКИ python
#------------------------------------------
import sys
import argparse
import re

#------------------------------------------
# БИБЛИОТЕКИ сторонние
#------------------------------------------

#------------------------------------------
# БИБЛИОТЕКА LU
#------------------------------------------

#------------------------------------------
# Разбор аргументов
#------------------------------------------
cSwitchChars = ('-', '/')

class TArgParser (argparse.ArgumentParser):
    """TArgParser"""
    luClassName = 'TArgParser'

    #--------------------------------------------------
    # constructor
    #--------------------------------------------------
    def __init__ (self, **kwargs):
        """ Constructor """
    #beginfunction
        super ().__init__ (**kwargs)
        self.__FArgs: argparse.Namespace = None
        self.__FArgsUnknown: argparse.Namespace = None
        self.__FArgsDICT = {}
        ...
    #endfunction

    #--------------------------------------------------
    # destructor
    #--------------------------------------------------
    def __del__ (self):
        """ destructor """
    #beginfunction
        LClassName = self.__class__.__name__
        s = '{} уничтожен'.format (LClassName)
        # LULog.LoggerTOOLS_AddLevel (LULog.DEBUGTEXT, s)
        #print (s)
    #endfunction

    #--------------------------------------------------
    # @property ArgParser
    #--------------------------------------------------
    # getter
    @property
    def ArgParser(self):
    #beginfunction
        return self
    #endfunction

    #--------------------------------------------------
    # @property ArgsDICT
    #--------------------------------------------------
    @property
    # getter
    def ArgsDICT (self):
    #beginfunction
        return self.__FArgsDICT
    #endfunction
    @ArgsDICT.setter
    def ArgsDICT (self, Value):
    #beginfunction
        self.__FArgsDICT = Value
    #endfunction

    #--------------------------------------------------
    # @property Args
    #--------------------------------------------------
    @property
    # getter
    def Args (self) -> argparse.Namespace:
    #beginfunction
        return self.__FArgs
    #endfunction
    @Args.setter
    def Args (self, Value: argparse.Namespace):
    #beginfunction
        self.__FArgs = Value
    #endfunction

    #--------------------------------------------------
    # @property ArgsUnknown
    #--------------------------------------------------
    @property
    # getter
    def ArgsUnknown (self) -> argparse.Namespace:
    #beginfunction
        return self.__FArgsUnknown
    #endfunction
    @ArgsUnknown.setter
    def ArgsUnknown (self, Value: argparse.Namespace):
    #beginfunction
        self.__FArgsUnknown = Value
    #endfunction

    def Clear (self):
    #beginfunction
        ...
    #endfunction

    def ReadARGS (self, AARGS: dict):

        def GetARG (AARGName, AARGValue):
        #beginfunction
            LResult = AARGS.get (AARGName).get (AARGValue)
            # try:
            #     LResult = AARGS [AARGName] [AARGValue]
            # except:
            #     LResult = None
            # #endtry
            return LResult
        #endfunction

    #beginfunction
        for name, value in AARGS.items ():
            # s = f"{name}={value}"
            #print (s)
            LArg = self.add_argument (GetARG (name, 'name'),
                                      dest = GetARG (name, 'dest'),
                                      type = GetARG (name, 'type'),
                                      default = GetARG (name, 'default'),
                                      help = GetARG (name, 'help'),
                                      action = GetARG (name, 'action'),
                                      choices = GetARG (name, 'choices')
                                      )
        #endfor

        # GArgParser.Args = GArgParser.ArgParser.parse_args (['-ld', '1'], namespace = C)

        # self.Args = self.ArgParser.parse_args ()
        self.Args, self.ArgsUnknown = self.parse_known_args ()
        #print (self.Args)

        self.ArgsDICT = vars (self.Args)
        #print (self.ArgsDICT)

        # for item in self.Args:
        #     ...
        # GArgParser.ArgsDICT = {'-ld': GArgParser.Args.ld, '-lf': GArgParser.Args.lf,
        #                    '-lfj': GArgParser.Args.lfj, '-ini': GArgParser.Args.ini}

    #endfunction
#endclass

"""
--------------------------------------
ArgumentParser objects
--------------------------------------
class argparse.ArgumentParser(prog=None, usage=None, description=None,
    epilog=None, parents=[], formatter_class=argparse.HelpFormatter,
    prefix_chars='-', fromfile_prefix_chars=None, argument_default=None,
    conflict_handler='error', add_help=True, allow_abbrev=True, exit_on_error=True)

    prog - имя программы (по умолчанию: os.path.basename(sys.argv[0]))
    usage - Строка, описывающая использование программы (по умолчанию: генерируется из аргументов, добавленных в парсер)
    description - текст для отображения перед аргументом help (по умолчанию без текста)
    epilog - текст для отображения после справки аргумента (по умолчанию без текста)
    parents - список объектов ArgumentParser, чьи аргументы также должны быть включены
    formatter_class - класс для настройки вывода справки
    prefix_chars - набор символов, которые предшествуют необязательным аргументам (по умолчанию: «-»).
    fromfile_prefix_chars - набор символов, которые префиксируют файлы,
        из которых должны быть прочитаны дополнительные аргументы (по умолчанию: нет)
    argument_default - глобальное значение по умолчанию для аргументов (по умолчанию: None)
    conflict_handler - стратегия разрешения конфликтующих опций (обычно ненужных)
    add_help - добавить параметр -h/--help в парсер (по умолчанию: True)
    allow_abbrev - позволяет сокращать длинные параметры, если аббревиатура недвусмысленна. (по умолчанию: Истина)
    exit_on_error - определяет, завершается ли ArgumentParser с информацией об ошибке при возникновении ошибки.
        (по умолчанию: Истина)
"""

"""
--------------------------------------
The add_argument() method
ArgumentParser.add_argument(name or flags...[, action][, nargs]
    [, const][, default][, type][, choices][, required][, help][, metavar][, dest])
--------------------------------------
    name or flags - либо имя, либо список строк параметров, например. foo или -f, --foo.
    action - основной тип действия, которое необходимо предпринять, когда этот аргумент встречается в командной строке.
        'store', 'store_const', 'store_true', 'append', 'append_const', 'count', 'help', 'version'
    nargs - количество аргументов командной строки, которые следует использовать.
        int, '?', '*', '+', or argparse.REMAINDER
    const - постоянное значение, необходимое для выбора некоторых действий и nargs.
    default - значение, создаваемое, если аргумент отсутствует в командной строке и если он
        отсутствует в объекте пространства имен. Defaults to None.
    type - Тип, в который должен быть преобразован аргумент командной строки.
        int, float, argparse.FileType('w'), or callable function
    choices - последовательность допустимых значений аргумента.
        ['foo', 'bar'], range(1, 10), or Container instance
    required - можно ли опустить параметр командной строки (только необязательные параметры).
        True or False
    help - Краткое описание того, что делает аргумент.
    metavar - имя аргумента в сообщениях об использовании.
    dest - имя атрибута, который будет добавлен к объекту, возвращаемому функцией parse_args().
    
    LArg = GArgParser.add_argument (
        '-ld', dest='ld',
        type=str, default='',
        help = 'log dir',
        
        action='store', required=True,
        # action='store_const', const=True
        # action='store_true',
        # action='store_false',

        nargs = '+',
        # nargs = '?', const='',

        # choices=[],
        metavar = 'LDM'
    )
"""

"""
--------------------------------------
The parse_args() method
--------------------------------------
ArgumentParser.parse_args(args=None, namespace=None)
    args - Список строк для разбора. Значение по умолчанию берется из sys.argv.
    namespace - Объект для получения атрибутов. По умолчанию используется новый пустой объект пространства имен.
----------------------------------
"""

#------------------------------------------------------
""" Работа со параметрами программы """
#------------------------------------------------------

#------------------------------------------------------
# __GetCmdLineArg
#------------------------------------------------------
def __GetCmdLineArg (ASwitch: str, ASwitchChars: (), AIgnoreCase: bool) -> str:
    """__GetCmdLineArg"""
#beginfunction
    i = 0
    for s in sys.argv:
        i = i + 1
        if len(ASwitchChars) == 0 or ((s[0] in ASwitchChars) and (len(s) > 1)):
            s = re.split('|'.join(ASwitchChars), s)[1]
            if AIgnoreCase:
                if s.upper() == ASwitch.upper():
                    if i < len(sys.argv):
                        s = sys.argv[i]
                        if s[0] in ASwitchChars:
                            return ''
                        else:
                            return s
                        #endif
                    #endif
                #endif
            else:
                if s == ASwitch:
                    i = i + 1
                    if i <= len(sys.argv):
                        s = sys.argv[i]
                        if s[0] in ASwitchChars:
                            return ''
                        else:
                            return s
                        #endif
                    #endif
                #endif
            #endif
    #endfor
    return ''
#endfunction

#------------------------------------------------------
# FindCmdLineSwitch
#------------------------------------------------------
def FindCmdLineSwitch (ASwitch: str, ASwitchChars: (), AIgnoreCase: bool) -> bool:
    """FindCmdLineSwitch"""
#beginfunction
    i = 0
    for s in sys.argv:
        i = i + 1
        if len(ASwitchChars) == 0 or ((s[0] in ASwitchChars) and (len(s) > 1)):
            s = re.split('|'.join(ASwitchChars), s)[1]
            if AIgnoreCase:
                if s.upper() == ASwitch.upper():
                    return True
                #endif
            else:
                if s == ASwitch:
                    return True
                #endif
            #endif
        #endif
    #endfor
    return False
#endfunction

#---------------------------------------------------------
# GetParam
#---------------------------------------------------------
def GetParam (AParamName: str, ADefaultValue: str) -> str:
    """GetParam"""
#beginfunction
    LResult = ADefaultValue
    if FindCmdLineSwitch (AParamName, cSwitchChars, True):
        LParamValue: str = __GetCmdLineArg (AParamName, cSwitchChars, True)
        LResult = LParamValue
    #endif
    return LResult
#endfunction

#---------------------------------------------------------
# CreateTArgParser
#---------------------------------------------------------
def CreateTArgParser (AProg: str, ADescrption: str) -> TArgParser:
    """CreateTArgParser"""
#beginfunction
    LResult = TArgParser (prog = AProg, description=ADescrption, prefix_chars='-/',
                          usage = None, epilog = None, parents = [], formatter_class = argparse.HelpFormatter,
                          fromfile_prefix_chars = None, argument_default = None,
                          conflict_handler = 'error', add_help = True)
    return LResult
#endfunction

GArgParser = CreateTArgParser ('LUArgParser', 'Параметры')

#---------------------------------------------------------
# main
#---------------------------------------------------------
def main ():
#beginfunction
    ...
#endfunction

#------------------------------------------
#
#------------------------------------------
#beginmodule
if __name__ == "__main__":
    main()
#endif

#endmodule



"""Some tests for the main module."""

from itertools import islice

from demo.main import fibo


def test_fibo(fibo_sequence_first_ten):
    """Test fibonacci function."""
    assert list(islice(fibo(), 10)) == fibo_sequence_first_ten



"""LUFile.py"""
# -*- coding: UTF-8 -*-
__annotations__ = """
 =======================================================
 Copyright (c) 2023-2024
 Author:
     Lisitsin Y.R.
 Project:
     LU_PY
     Python (LU)
 Module:
     LUFile.py

 =======================================================
"""

import errno
#------------------------------------------
# БИБЛИОТЕКИ python
#------------------------------------------
import os
import stat
import datetime
import logging
import tempfile
import re
import ctypes
import pathlib

# if platform.system() == 'Windows':
#     import win32api
#     print('Windows')
#     import win32con
# #endif
# if platform.system() == 'Linux':
#     ...
# #endif

#------------------------------------------
# БИБЛИОТЕКИ сторонние
#------------------------------------------
import shutil
import chardet

#------------------------------------------
# БИБЛИОТЕКА LU
#------------------------------------------
import lyrpy.LUStrDecode as LUStrDecode
import lyrpy.LUDateTime as LUDateTime
import lyrpy.LUos as LUos
import lyrpy.LULog as LULog

"""
#--------------------------------------------------------------------------------
f = open(file_name, access_mode, encoding='')
    file_name = имя открываемого файла
    access_mode = режим открытия файла. Он может быть: для чтения, записи и т. д.
    По умолчанию используется режим чтения (r), если другое не указано.
    Далее полный список режимов открытия файла
Режим   Описание
r   Только для чтения.
w   Только для записи. Создаст новый файл, если не найдет с указанным именем.
rb  Только для чтения (бинарный).
wb  Только для записи (бинарный). Создаст новый файл, если не найдет с указанным именем.
r+  Для чтения и записи.
rb+ Для чтения и записи (бинарный).
w+  Для чтения и записи. Создаст новый файл для записи, если не найдет с указанным именем.
wb+ Для чтения и записи (бинарный). Создаст новый файл для записи, если не найдет с указанным именем.
a   Откроет для добавления нового содержимого. Создаст новый файл для записи, если не найдет с указанным именем.
a+  Откроет для добавления нового содержимого. Создаст новый файл для чтения записи, если не найдет с указанным именем.
ab  Откроет для добавления нового содержимого (бинарный). Создаст новый файл для записи, если не найдет с указанным именем.
ab+ Откроет для добавления нового содержимого (бинарный). Создаст новый файл для чтения записи, если не найдет с указанным именем.    

# LFile = open (AFileName, 'r', encoding='utf-8')
# LFile = open (AFileName, 'r', encoding='cp1251')
#--------------------------------------------------------------------------------
"""

# cDefaultEncoding = 'cp1251'
cDefaultEncoding = 'utf-8'

#--------------------------------------------------------------------------------
# DirectoryExists
#--------------------------------------------------------------------------------
def DirectoryExists (APath: str) -> bool:
    """DirectoryExists """
#beginfunction
    return os.path.isdir(APath)
#endfunction

#--------------------------------------------------------------------------------
# ForceDirectories
#--------------------------------------------------------------------------------
def ForceDirectories (ADir: str) -> bool:
    """ForceDirectories"""
#beginfunction
    try:
        os.makedirs (ADir, exist_ok = True)
    except:
        s = f'Unable to create directory {ADir:s} ...'
        # LULog.LoggerTOOLS_AddLevel(logging.error, s)
        LULog.LoggerAdd(LULog.LoggerTOOLS, logging.error, s)
    #endtry
    LResult = DirectoryExists (ADir)
    return LResult
#endfunction

#--------------------------------------------------------------------------------
# GetDirectoryTreeSize
#--------------------------------------------------------------------------------
def GetDirectoryTreeSize(ADir: str) -> int:
    """GetDirectoryTreeSize"""
    """Return total size of files in given path and subdirs"""
#beginfunction
    Ltotal = 0
    for Lentry in os.scandir(ADir):
        if Lentry.is_dir(follow_symlinks=False):
            Ltotal += GetDirectoryTreeSize(Lentry.path)
        else:
            Ltotal += Lentry.stat(follow_symlinks=False).st_size
    return Ltotal
#endfunction

#--------------------------------------------------------------------------------
# DeleteDirectoryTree
#--------------------------------------------------------------------------------
def DeleteDirectoryTree (ADir: str) -> bool:
    """DeleteDirectoryTree"""
    """
    Удалить дерево каталогов в Windows,
    где для некоторых файлов установлен бит только для чтения.
    Он использует обратный вызов onerror, чтобы очистить бит readonly и повторить попытку удаления.
    """
    def remove_readonly (func, path, _):
        """remove_readonly"""
    #beginfunction
        Ls = f'Clear the readonly bit and reattempt the removal {path:s} ...'
        # LULog.LoggerTOOLS_AddLevel(logging.DEBUG, Ls)
        LULog.LoggerAdd(LULog.LoggerTOOLS, logging.DEBUG, Ls)
        os.chmod (path, stat.S_IWRITE)
        func (path)
    #endfunction

    def errorRemoveReadonly (func, path, exc):
        """errorRemoveReadonly"""
    #beginfunction
        excvalue = exc [1]
        if func in (os.rmdir, os.remove) and excvalue.errno == errno.EACCES:
            # change the file to be readable,writable,executable: 0777
            os.chmod (path, stat.S_IRWXU | stat.S_IRWXG | stat.S_IRWXO)
            # retry
            func (path)
        else:
            # raiseenter code here
            ...
        #endif
    #endfunction

#beginfunction
    LResult = True
    if DirectoryExists (ADir):
        s = f'DeleteDirectoryTree {ADir:s} ...'
        # LULog.LoggerTOOLS_AddLevel(logging.DEBUG, s)
        LULog.LoggerAdd(LULog.LoggerTOOLS, logging.DEBUG, s)
        try:
            # shutil.rmtree (ADirectoryName, ignore_errors = True, onexc = None)
            shutil.rmtree (ADir, ignore_errors = False, onerror = remove_readonly)
            LResult = True
        except:
            s = f'Unable delete directory {ADir:s} ...'
            # LULog.LoggerTOOLS_AddLevel (logging.error, s)
            LULog.LoggerAdd (LULog.LoggerTOOLS, logging.error, s)
            LResult = False
        #endtry
    #endif
    return LResult
#endfunction

#--------------------------------------------------------------------------------
# DeleteDirectory_walk
#--------------------------------------------------------------------------------
# Delete everything reachable from the directory named in 'top',
# assuming there are no symbolic links.
# CAUTION:  This is dangerous!  For example, if top == '/', it
# could delete all your disk files.
#--------------------------------------------------------------------------------
def DirectoryClear (ADir: str) -> bool:
    """DirectoryClear"""
#beginfunction
    LResult = True
    if DirectoryExists (ADir):
        for root, dirs, files in os.walk (ADir, topdown = False):
            for file in files:
                os.remove (os.path.join (root, file))
            #endfor
            for ldir in dirs:
                os.rmdir (os.path.join (root, ldir))
            #endfor
        #endfor
        LResult = True
    #endif
    return LResult
#endfunction

#--------------------------------------------------------------------------------
# FileExists
#--------------------------------------------------------------------------------
def FileExists (AFileName: str) -> bool:
    """FileExists"""
#beginfunction
    return os.path.isfile(AFileName)
#endfunction

#--------------------------------------------------------------------------------
# GetFileDateTime
#--------------------------------------------------------------------------------
def GetFileDateTime (AFileName: str) -> ():
    """GetFileDateTime"""
#beginfunction
    LTuple = ()
    LFileTimeCreate = 0
    LFileTimeMod = 0
    LFileTimeCreateDate = 0
    LFileTimeModDate = 0
    if FileExists (AFileName):
        # file creation
        LFileTimeCreate: datetime = os.path.getctime (AFileName)
        # file modification
        LFileTimeMod: datetime = os.path.getmtime (AFileName)
        # convert creation timestamp into DateTime object
        LFileTimeCreateDate: datetime = datetime.datetime.fromtimestamp (LFileTimeCreate)
        # convert timestamp into DateTime object
        LFileTimeModDate: datetime = datetime.datetime.fromtimestamp (LFileTimeMod)
    #endif
    LTuple = (LFileTimeMod, LFileTimeCreate, LFileTimeModDate, LFileTimeCreateDate)
    return LTuple
#endfunction

#--------------------------------------------------------------------------------
# GetDirDateTime
#--------------------------------------------------------------------------------
def GetDirDateTime (AFileName: str) -> ():
    """GetDirDateTime"""
#beginfunction
    LTuple = ()
    LFileTimeCreate = 0
    LFileTimeMod = 0
    LFileTimeCreateDate = 0
    LFileTimeModDate = 0
    if DirectoryExists (AFileName):
        # file creation
        LFileTimeCreate: datetime = os.path.getctime (AFileName)
        # file modification
        LFileTimeMod: datetime = os.path.getmtime (AFileName)
        # convert creation timestamp into DateTime object
        LFileTimeCreateDate: datetime = datetime.datetime.fromtimestamp (LFileTimeCreate)
        # convert timestamp into DateTime object
        LFileTimeModDate: datetime = datetime.datetime.fromtimestamp (LFileTimeMod)
    #endif
    LTuple = (LFileTimeMod, LFileTimeCreate, LFileTimeModDate, LFileTimeCreateDate)
    return LTuple
#endfunction

def cmptimestamps(AFileNameSource: str, AFileNameDest: str, _use_ctime):
    """ Compare time stamps of two files and return True
    if file1 (source) is more recent than file2 (target) """
#beginfunction
    st1 = os.stat (AFileNameSource)
    st2 = os.stat (AFileNameDest)
    mtime_cmp = int((st1.st_mtime - st2.st_mtime) * 1000) > 0
    if _use_ctime:
        return mtime_cmp or int((AFileNameSource.st_ctime - AFileNameDest.st_mtime) * 1000) > 0
    else:
        return mtime_cmp
    #endif
#endfunction

#--------------------------------------------------------------------------------
# COMPAREFILETIMES
#--------------------------------------------------------------------------------
def COMPAREFILETIMES (AFileNameSource: str, AFileNameDest: str) -> int:
    """COMPAREFILETIMES"""
#beginfunction
    if not FileExists (AFileNameSource):
        #-2 File1 could not be opened (see @ERROR for more information).
        return -2
    #endif
    if not FileExists (AFileNameDest):
        #-3 File2 could not be opened (see @ERROR for more information).
        return -3
    #endif

    LFileName1m = GetFileDateTime (AFileNameSource)[0]
    LFileName2m = GetFileDateTime (AFileNameDest)[0]
    LFileName1c = GetFileDateTime (AFileNameSource)[0]
    LFileName2c = GetFileDateTime (AFileNameDest)[0]

    if LFileName1m == LFileName2m:
        #0 File1 and file2 have the same date and time.
        return 0
    else:
        if LFileName1m > LFileName2m:
            #1 File1 is more recent than file2.
            return 1
        else:
            #-1 File1 is older than file2.
            return -1
        #endif
    #endif
    #------------------------------------------------------------------------------
    # if int ((LFileName1m - LFileName2m) * 1000) == 0:
    #     return 0
    # else:
    #     if int ((LFileName1m - LFileName2m) * 1000) > 0:
    #         return 1
    #     else:
    #         return -1
    #     #endif
    # #endif
    #------------------------------------------------------------------------------
#endfunction

#--------------------------------------------------------------------------------
# CheckFileExt
#--------------------------------------------------------------------------------
def CheckFileExt (AFileName: str, AExt: str) -> bool:
    """CheckFileExt"""
#beginfunction
    if AExt != "":
        LResult = ExtractFileName(AFileName).endswith(AExt)
    else:
        LResult = False
    #endif
    return LResult
#endfunction

#--------------------------------------------------------------------------------
# GetFileSize
#--------------------------------------------------------------------------------
def GetFileSize (AFileName: str) -> int:
    """GetFileSize"""
#beginfunction
    if FileExists (AFileName):
        LResult = os.path.getsize (AFileName)
    else:
        LResult = 0
    #endif
    return LResult
#endfunction

#--------------------------------------------------------------------------------
# ExpandFileName
#--------------------------------------------------------------------------------
def ExpandFileName (APath: str) -> str:
    """ExpandFileName"""
#beginfunction
    LResult = os.path.abspath(APath)
    return LResult
#endfunction

#--------------------------------------------------------------------------------
# ExtractFileDir
#--------------------------------------------------------------------------------
def ExtractFileDir (APath: str) -> str:
    """ExtractFileDir"""
#beginfunction
    LDir, LFileName = os.path.split(APath)
    return LDir
#endfunction

#--------------------------------------------------------------------------------
# ExtractFileName
#--------------------------------------------------------------------------------
def ExtractFileName (APath: str) -> str:
    """ExtractFileName"""
#beginfunction
    LPath, LFileName = os.path.split(APath)
    return LFileName
#endfunction

#-------------------------------------------------------------------------------
# ExtractFileNameWithoutExt
#-------------------------------------------------------------------------------
def ExtractFileNameWithoutExt (AFileName: str) -> str:
    """ExtractFileNameWithoutExt"""
#beginfunction
    LResult = os.path.basename (AFileName).split ('.') [0]
    return LResult
#endfunction

#--------------------------------------------------------------------------------
# ExtractFileExt
#--------------------------------------------------------------------------------
def ExtractFileExt (AFileName: str) -> str:
    """ExtractFileExt"""
#beginfunction
    LResult = os.path.basename(AFileName)
    LFileName, LFileExt = os.path.splitext(LResult)
    return LFileExt
#endfunction

#---------------------------------------------------------------------------------------------
# GetFileDir (APath: str) -> str:
#---------------------------------------------------------------------------------------------
def GetFileDir (APath: str) -> str:
    """GetFileDir"""
#beginfunction
    return ExtractFileDir (APath)
#endfunction

#--------------------------------------------------------------------------------
# GetFileName (APath: str) -> str:
#--------------------------------------------------------------------------------
def GetFileName (APath: str) -> str:
    """GetFileName"""
#beginfunction
    return ExtractFileNameWithoutExt (APath)
#endfunction

#-------------------------------------------------------------------------------
# GetFileNameWithoutExt (AFileName: str) -> str:
#-------------------------------------------------------------------------------
def GetFileNameWithoutExt (AFileName: str) -> str:
    """GetFileNameWithoutExt"""
#beginfunction
    return ExtractFileNameWithoutExt (AFileName)
#endfunction

#---------------------------------------------------------------------------------------------
# GetFileExt (AFileName: str) -> str:
#---------------------------------------------------------------------------------------------
def GetFileExt (AFileName: str) -> str:
    """GetFileExt"""
#beginfunction
    return ExtractFileExt (AFileName)
#endfunction

#--------------------------------------------------------------------------------
# GetFileEncoding (AFileName: str) -> str:
#--------------------------------------------------------------------------------
def GetFileEncoding (AFileName: str) -> str:
    """GetFileEncoding"""
#beginfunction
    LEncoding = ''
    if FileExists(AFileName):
        LFile = open (AFileName, 'rb')
        LRawData = LFile.read ()
        LResult = chardet.detect (LRawData)
        LEncoding = LResult ['encoding']
        LFile.close ()
    #endif
    return LEncoding
#endfunction

#--------------------------------------------------------------------------------
# IncludeTrailingBackslash
#--------------------------------------------------------------------------------
def IncludeTrailingBackslash (APath: str) -> str:
    """IncludeTrailingBackslash"""
#beginfunction
    LResult = APath.rstrip('\\')+'\\'
    # LResult = pathlib.WindowsPath (APath)
    # LResult = APath.rstrip('/')+'/'
    return LResult
#endfunction

#--------------------------------------------------------------------------------
# GetDirNameYYMMDD
#--------------------------------------------------------------------------------
def GetDirNameYYMMDD (ARootDir: str, ADate: datetime.datetime) -> str:
    """GetDirNameYYMMDD"""
#beginfunction
    LYMDStr: str = LUDateTime.DateTimeStr(False, ADate, LUDateTime.cFormatDateYYMMDD_02, False)
    LResult = IncludeTrailingBackslash(ARootDir)+LYMDStr
    return LResult
#endfunction

#--------------------------------------------------------------------------------
# GetDirNameYYMM
#--------------------------------------------------------------------------------
def GetDirNameYYMM (ARootDir: str, ADate: datetime.datetime) -> str:
    """GetDirNameYYMM"""
#beginfunction
    LYMDStr: str = LUDateTime.DateTimeStr(False, ADate, LUDateTime.cFormatDateYYMM_02, False)
    LResult = IncludeTrailingBackslash(ARootDir)+LYMDStr
    return LResult
#endfunction

#--------------------------------------------------------------------------------
# GetTempDir
#--------------------------------------------------------------------------------
def GetTempDir () -> str:
    """GetTempDir"""
#beginfunction
    # LResult = win32api.GetTempPath()
    LResult = tempfile.gettempdir ()
    print('TEMP:',LResult)
    return LResult
#endfunction

#-------------------------------------------------------------------------------
# GetFileAttrStr
#-------------------------------------------------------------------------------
def GetFileAttrStr (Aattr: int) -> str:
    """GetFileAttrStr"""
#beginfunction
    #-------------------------------------------------------------------------------
    #                                        0x      00       00       20       20
    #                                        0b00000000 00000000 00100000 00100000
    #-------------------------------------------------------------------------------
    #stat.FILE_ATTRIBUTE_NO_SCRUB_DATA       0b00000000 00000010 00000000 00000000
    #stat.FILE_ATTRIBUTE_VIRTUAL             0b00000000 00000001 00000000 00000000

    #stat.FILE_ATTRIBUTE_INTEGRITY_STREAM    0b00000000 00000000 10000000 00000000
    #stat.FILE_ATTRIBUTE_ENCRYPTED           0b00000000 00000000 01000000 00000000
    #stat.FILE_ATTRIBUTE_NOT_CONTENT_INDEXED 0b00000000 00000000 00100000 00000000
    #stat.FILE_ATTRIBUTE_OFFLINE             0b00000000 00000000 00010000 00000000
    #stat.FILE_ATTRIBUTE_COMPRESSED          0b00000000 00000000 00001000 00000000
    #stat.FILE_ATTRIBUTE_REPARSE_POINT       0b00000000 00000000 00000100 00000000
    #stat.FILE_ATTRIBUTE_SPARSE_FILE         0b00000000 00000000 00000010 00000000
    #stat.FILE_ATTRIBUTE_TEMPORARY           0b00000000 00000000 00000001 00000000

    #stat.FILE_ATTRIBUTE_NORMAL              0b00000000 00000000 00000000 10000000
    #stat.FILE_ATTRIBUTE_DEVICE              0b00000000 00000000 00000000 01000000
    #-------------------------------------------------------------------------------
    #stat.FILE_ATTRIBUTE_ARCHIVE             0b00000000 00000000 00000000 00100000
    #-------------------------------------------------------------------------------
    #stat.FILE_ATTRIBUTE_DIRECTORY           0b00000000 00000000 00000000 00010000
    #-------------------------------------------------------------------------------
    #stat.                                   0b00000000 00000000 00000000 00001000
    #-------------------------------------------------------------------------------
    #stat.FILE_ATTRIBUTE_SYSTEM              0b00000000 00000000 00000000 00000100
    #-------------------------------------------------------------------------------
    #stat.FILE_ATTRIBUTE_HIDDEN              0b00000000 00000000 00000000 00000010
    #-------------------------------------------------------------------------------
    #stat.FILE_ATTRIBUTE_READONLY            0b00000000 00000000 00000000 00000001
    #-------------------------------------------------------------------------------
    Lattr = Aattr
    sa = ''
    sa += '????????'
    sa += '1' if Lattr & 0b100000000000000000000000 else '?'
    sa += '1' if Lattr & 0b010000000000000000000000 else '?'
    sa += '1' if Lattr & 0b001000000000000000000000 else '?'
    sa += '1' if Lattr & 0b000100000000000000000000 else '?'
    sa += '1' if Lattr & 0b000010000000000000000000 else '?'
    sa += '1' if Lattr & 0b000001000000000000000000 else '?'
    sa += '1' if Lattr & stat.FILE_ATTRIBUTE_NO_SCRUB_DATA else '.'
    sa += '1' if Lattr & stat.FILE_ATTRIBUTE_VIRTUAL else '.'

    sa += '1' if Lattr & stat.FILE_ATTRIBUTE_INTEGRITY_STREAM else '.'
    sa += '1' if Lattr & stat.FILE_ATTRIBUTE_ENCRYPTED else '.'
    sa += '1' if Lattr & stat.FILE_ATTRIBUTE_NOT_CONTENT_INDEXED else '.'
    sa += '1' if Lattr & stat.FILE_ATTRIBUTE_OFFLINE else '.'
    sa += '1' if Lattr & stat.FILE_ATTRIBUTE_COMPRESSED else '.'
    sa += '1' if Lattr & stat.FILE_ATTRIBUTE_REPARSE_POINT else '.'
    sa += '1' if Lattr & stat.FILE_ATTRIBUTE_SPARSE_FILE else '.'
    sa += '1' if Lattr & stat.FILE_ATTRIBUTE_TEMPORARY else '.'

    sa += '1' if Lattr & stat.FILE_ATTRIBUTE_NORMAL else '.'
    sa += '1' if Lattr & stat.FILE_ATTRIBUTE_DEVICE else '.'
    sa += 'a' if Lattr & stat.FILE_ATTRIBUTE_ARCHIVE else '.'
    sa += 'd' if Lattr & stat.FILE_ATTRIBUTE_DIRECTORY else '.'
    sa += '.'
    sa += 's' if Lattr & stat.FILE_ATTRIBUTE_SYSTEM else '.'
    sa += 'h' if Lattr & stat.FILE_ATTRIBUTE_HIDDEN else '.'
    sa += 'r' if Lattr & stat.FILE_ATTRIBUTE_READONLY else '.'
    return sa
#endfunction

#-------------------------------------------------------------------------------
# GetFileModeStrUnix
#-------------------------------------------------------------------------------
def GetFileModeStrUnix (Amode: int) -> str:
    """GetFileModeStrUnix"""
#beginfunction
    # chmod(path,mode)
    # s = f'stat.S_ISUID: {bin (stat.S_ISUID):s}'
    # LULog.LoggerTOOLS_AddLevel (LULog.TEXT, s)
    # s = f'stat.S_ISGID: {bin (stat.S_ISGID):s}'
    # LULog.LoggerTOOLS_AddLevel (LULog.TEXT, s)
    # s = f'stat.S_ENFMT: {bin (stat.S_ENFMT):s}'
    # LULog.LoggerTOOLS_AddLevel (LULog.TEXT, s)
    # s = f'stat.S_ISVTX: {bin (stat.S_ISVTX):s}'
    # LULog.LoggerTOOLS_AddLevel (LULog.TEXT, s)
    # s = f'stat.S_IREAD: {bin (stat.S_IREAD):s}'
    # LULog.LoggerTOOLS_AddLevel (LULog.TEXT, s)
    # s = f'stat.S_IWRITE: {bin (stat.S_IWRITE):s}'
    # LULog.LoggerTOOLS_AddLevel (LULog.TEXT, s)
    # s = f'stat.S_IEXEC: {bin (stat.S_IEXEC):s}'
    # LULog.LoggerTOOLS_AddLevel (LULog.TEXT, s)
    # s = f'stat.S_IRWXU: {bin (stat.S_IRWXU):s}'
    # LULog.LoggerTOOLS_AddLevel (LULog.TEXT, s)
    # s = f'stat.S_IRUSR: {bin (stat.S_IRUSR):s}'
    # LULog.LoggerTOOLS_AddLevel (LULog.TEXT, s)
    # s = f'stat.S_IWUSR: {bin (stat.S_IWUSR):s}'
    # LULog.LoggerTOOLS_AddLevel (LULog.TEXT, s)
    # s = f'stat.S_IXUSR: {bin (stat.S_IXUSR):s}'
    # LULog.LoggerTOOLS_AddLevel (LULog.TEXT, s)
    # s = f'stat.S_IRWXG: {bin (stat.S_IRWXG):s}'
    # LULog.LoggerTOOLS_AddLevel (LULog.TEXT, s)
    # s = f'stat.S_IRGRP: {bin (stat.S_IRGRP):s}'
    # LULog.LoggerTOOLS_AddLevel (LULog.TEXT, s)
    # s = f'stat.S_IWGRP: {bin (stat.S_IWGRP):s}'
    # LULog.LoggerTOOLS_AddLevel (LULog.TEXT, s)
    # s = f'stat.S_IXGRP: {bin (stat.S_IXGRP):s}'
    # LULog.LoggerTOOLS_AddLevel (LULog.TEXT, s)
    # s = f'stat.S_IRWXO: {bin (stat.S_IRWXO):s}'
    # LULog.LoggerTOOLS_AddLevel (LULog.TEXT, s)
    # s = f'stat.S_IROTH: {bin (stat.S_IROTH):s}'
    # LULog.LoggerTOOLS_AddLevel (LULog.TEXT, s)
    # s = f'stat.S_IWOTH: {bin (stat.S_IWOTH):s}'
    # LULog.LoggerTOOLS_AddLevel (LULog.TEXT, s)
    # s = f'stat.S_IXOTH: {bin (stat.S_IXOTH):s}'
    # LULog.LoggerTOOLS_AddLevel (LULog.TEXT, s)

    #-------------------------------------------------------------------------------
    # stat.S_ISUID − Set user ID on execution
    # stat.S_ISUID:  0b00010000 00000000
    # stat.S_ISGID − Set group ID on execution
    # stat.S_ISGID:  0b00000100 00000000
    # stat.S_ENFMT – Enforced record locking
    # stat.S_ENFMT:  0b00000100 00000000
    # stat.S_ISVTX – After execution, save text image
    # stat.S_ISVTX:  0b00000010 00000000
    #-------------------------------------------------------------------------------
    # stat.S_IREAD − Read by owner
    # stat.S_IREAD:  0b00000001 00000000
    # stat.S_IWRITE − Write by owner
    # stat.S_IWRITE: 0b00000000 10000000
    # stat.S_IEXEC − Execute by owner
    # stat.S_IEXEC:  0b00000000 01000000
    #-------------------------------------------------------------------------------
    # stat.S_IRWXU − Read, write, and execute by owner
    # stat.S_IRWXU:  0b00000001 11000000 Owner
    #-------------------------------------------------------------------------------
    # stat.S_IRUSR − Read by owner
    # stat.S_IRUSR:  0b00000001 00000000
    # stat.S_IWUSR − Write by owner
    # stat.S_IWUSR:  0b00000000 10000000
    # stat.S_IXUSR − Execute by owner
    # stat.S_IXUSR:  0b00000000 01000000
    #-------------------------------------------------------------------------------
    # stat.S_IRWXG − Read, write, and execute by group
    # stat.S_IRWXG:  0b00000000 00111000 Group
    #-------------------------------------------------------------------------------
    # stat.S_IRGRP − Read by group
    # stat.S_IRGRP:  0b00000000 00100000
    # stat.S_IWGRP − Write by group
    # stat.S_IWGRP:  0b00000000 00010000
    # stat.S_IXGRP − Execute by group
    # stat.S_IXGRP:  0b00000000 00001000
    #-------------------------------------------------------------------------------
    # stat.S_IRWXO − Read, write, and execute by others
    # stat.S_IRWXO:  0b00000000 00000111 Others
    #-------------------------------------------------------------------------------
    # stat.S_IROTH − Read by others
    # stat.S_IROTH:  0b00000000 00000100
    # stat.S_IWOTH − Write by others
    # stat.S_IWOTH:  0b00000000 00000010
    # stat.S_IXOTH − Execute by others
    # stat.S_IXOTH:  0b00000000 00000001
    #-------------------------------------------------------------------------------
    Lmode = Amode
    sa = ''
    sa += '1' if Lmode & 0b1000000000000000 else '-'
    sa += '1' if Lmode & 0b0100000000000000 else '-'
    sa += '1' if Lmode & 0b0010000000000000 else '-'

    sa += '1' if Lmode & stat.S_ISUID else '-'
    sa += '1' if Lmode & stat.S_ISGID else '-'
    sa += '1' if Lmode & stat.S_ENFMT else '-'
    sa += '1' if Lmode & stat.S_ISVTX else '-'
    #-------------------------------------------------------------------------------
    sa += 'r' if Lmode & stat.S_IRUSR else '-'
    sa += 'w' if Lmode & stat.S_IWUSR else '-'
    sa += 'x' if Lmode & stat.S_IXUSR else '-'
    #-------------------------------------------------------------------------------
    sa += 'r' if Lmode & stat.S_IRGRP else '-'
    sa += 'w' if Lmode & stat.S_IWGRP else '-'
    sa += 'x' if Lmode & stat.S_IXGRP else '-'
    #-------------------------------------------------------------------------------
    sa += 'r' if Lmode & stat.S_IROTH else '-'
    sa += 'w' if Lmode & stat.S_IWOTH else '-'
    sa += 'x' if Lmode & stat.S_IXOTH else '-'
    return sa
#endfunction

#-------------------------------------------------------------------------------
# GetFileAttr
#-------------------------------------------------------------------------------
def GetFileAttr (AFileName: str) -> int:
    """GetFileAttr"""
#beginfunction
    s = f'GetFileAttr: {AFileName:s}'
    # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
    LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)

    LResult = 0

    if FileExists (AFileName) or DirectoryExists (AFileName):
        LStat = os.stat (AFileName)

        LOSInfo = LUos.TOSInfo ()
        match LOSInfo.system:
            case 'Windows':
                Lmode = LStat.st_mode
                s = f'Lmode: {Lmode:d} {hex (Lmode):s} {bin (Lmode):s} {stat.filemode (Lmode):s}'
                # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
                LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
                Lattr = LStat.st_file_attributes

                # Lattr = win32api.GetFileAttributes (AFileName)

                s = f'Lattr:{Lattr:d} {hex (Lattr):s} {bin (Lattr):s} {GetFileAttrStr (Lattr):s}'
                # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
                LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
                LResult = Lattr
            case 'Linux':
                Lmode = LStat.st_mode
                s = f'Lmode:{Lmode:d} {hex (Lmode):s} {bin (Lmode):s} {GetFileModeStrUnix (Lmode):s}'
                # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
                LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
                LResult = Lmode
            case _:
                s = f'Неизвестная система ...'
                # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
                LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
                LResult = 0
        #endmatch
    #endif
    return LResult
#endfunction

#-------------------------------------------------------------------------------
# SetFileAttr
#-------------------------------------------------------------------------------
def SetFileAttr (AFileName: str, Aattr: int, AClear: bool):
    """SetFileAttr"""
#beginfunction
    s = f'SetFileAttr: {Aattr:d} {hex (Aattr):s} {bin (Aattr):s}'
    # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
    LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)

    LOSInfo = LUos.TOSInfo ()
    match LOSInfo.system:
        case 'Windows':
            Lattr = GetFileAttr(AFileName)
            s = f'Lattr - current: {Lattr:d} {hex (Lattr):s} {bin (Lattr):s}'
            # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
            LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)

            if AClear:
                LattrNew = Lattr & ~Aattr
                s = f'[clear]: {bin (LattrNew):s} {LattrNew:d} {hex (LattrNew):s} {bin (LattrNew):s}'
                # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
                LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
            else:
                LattrNew = Lattr | ~Aattr
                s = f'[set]: {bin (LattrNew):s} {LattrNew:d} {hex (LattrNew):s} {bin (LattrNew):s}'
                # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
                LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
            #endif

            # if os.path.isdir (AFileName):
            #     LResult = ctypes.windll.kernel32.SetFileAttributesW (AFileName, LattrNew)
            # else:
            #     win32api.SetFileAttributes (AFileName, LattrNew)
            # #endif

            LResult = ctypes.windll.kernel32.SetFileAttributesW (AFileName, LattrNew)
        case 'Linux':
            raise NotImplementedError('SetFileAttr Linux not implemented ...')
        case _:
            s = f'Неизвестная система ...'
            # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
            LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
    #endmatch
#endfunction

#-------------------------------------------------------------------------------
# SetFileMode
#-------------------------------------------------------------------------------
def SetFileMode (AFileName: str, Amode: int, AClear: bool, Aflags: int):
    """SetFileMode"""
#beginfunction
    s = f'SetFileMode: {Amode:d} {hex (Amode):s} {bin (Amode):s}'
    # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
    LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)

    Lattr = 0

    LOSInfo = LUos.TOSInfo ()
    match LOSInfo.system:
        case 'Windows':
            # Change the file's permissions to writable
            # os.chmod (AFileName, os.W_OK)
            ...
        case 'Linux':
            # os.chflags() method in Python used to set the flags of path to the numeric flags;
            # available in Unix only
            # os.UF_HIDDEN
            if AClear:
                LattrNew = Lattr & ~Aflags
                s = f'SetFileAttr [clear]: {bin (Aflags):s} {LattrNew:d} {hex (LattrNew):s} {bin (LattrNew):s}'
                # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
                LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
            else:
                LattrNew = Lattr | Aflags
                s = f'SetFileAttr [set]: {bin (Aflags):s}{LattrNew:d} {hex (LattrNew):s} {bin (LattrNew):s}'
                # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
                LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
            #endif
            os.chflags (AFileName, Aflags)
        case _:
            s = f'Неизвестная система ...'
            # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
            LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
    #endmatch
#endfunction

#-------------------------------------------------------------------------------
# SetFileFlags
#-------------------------------------------------------------------------------
def SetFileFlags (AFileName: str, Aflags: int, AClear: bool):
    """SetFileMode"""
#beginfunction
    s = f'SetFileMode: {Aflags:d} {hex (Aflags):s} {bin (Aflags):s}'
    # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
    LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)

    LOSInfo = LUos.TOSInfo ()
    match LOSInfo.system:
        case 'Windows':
            raise NotImplementedError('SetFileAttr Windows not implemented...')
        case 'Linux':
            # os.chflags() method in Python used to set the flags of path to the numeric flags;
            # available in Unix only
            # os.UF_HIDDEN

            Lattr = 0

            if AClear:
                LflagsNew = Lattr & ~Aflags
                s = f'[clear]: {bin (LflagsNew):s} {LflagsNew:d} {hex (LflagsNew):s} {bin (LflagsNew):s}'
                # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
                LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
            else:
                LflagsNew = Lattr | ~Aflags
                s = f'[set]: {bin (LflagsNew):s} {LflagsNew:d} {hex (LflagsNew):s} {bin (LflagsNew):s}'
                # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
                LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
            #endif
            os.chflags (AFileName, LflagsNew)
        case _:
            s = f'Неизвестная система ...'
            # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
            LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
    #endmatch
#endfunction

#-------------------------------------------------------------------------------
# FileDelete
#-------------------------------------------------------------------------------
def FileDelete (AFileName: str) -> bool:
    """FileDelete"""
#beginfunction
    s = f'FileDelete: {AFileName:s}'

    # LULog.LoggerTOOLS_setLevel(logging.INFO)
    # LULog.LoggerTOOLS_setLevel(logging.DEBUG)

    # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
    LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
    LResult = True

    if FileExists (AFileName):
        LOSInfo = LUos.TOSInfo ()
        match LOSInfo.system:
            case 'Windows':
                try:
                    Lattr = GetFileAttr (AFileName)
                    if Lattr & stat.FILE_ATTRIBUTE_READONLY:
                        s = f'Clear ReadOnly ...'
                        # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
                        LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
                        SetFileAttr (AFileName, stat.FILE_ATTRIBUTE_READONLY, True)

                        # FileSetAttr (FileName, FileGetAttr(FileName) and (faReadOnly xor $FF));
                        # Change the file's permissions to writable
                        # os.chmod (AFileName, os.W_OK)

                    #endif
                    os.remove (AFileName)
                    LResult = True
                except:
                    s = f'ERROR: FileDelete ...'
                    # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
                    LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
                    LResult = False
                #endtry
            case 'Linux':
                raise NotImplementedError('FileDelete Linux not implemented...')
            case _:
                s = f'Неизвестная система ...'
                # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
                LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
        #endmatch
    #endif
    return LResult
#endfunction

#-------------------------------------------------------------------------------
# FileCopy
#-------------------------------------------------------------------------------
def FileCopy (AFileNameSource: str, AFileNameDest: str, Overwrite: bool) -> bool:
    """FileCopy"""
#beginfunction
    s = f'FileCopy: {AFileNameSource:s} -> {AFileNameDest:s}'
    # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
    LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)

    LResult = True

    if FileExists (AFileNameSource):

        LDestPath = ExtractFileDir (AFileNameDest)
        if not DirectoryExists (LDestPath):
            ForceDirectories (LDestPath)
        #endif

        LOSInfo = LUos.TOSInfo ()
        match LOSInfo.system:
            case 'Windows':
                try:
                    # Функция shutil.copy() копирует данные файла и режима доступа к файлу.
                    # Другие метаданные, такие как время создания и время изменения файла не сохраняются.
                    # Чтобы сохранить все метаданные файла из оригинала, используйте функцию shutil.copy2().

                    # LResult = shutil.copy (AFileNameSource, AFileNameDest) != ''

                    # LResult = True
                    LResult = shutil.copy2 (AFileNameSource, AFileNameDest) != ''
                    # LResult = shutil.copy2 (AFileNameSource, LDestPath) != ''
                    # shutil.copystat (AFileNameSource, AFileNameDest)

                except:
                    s = f'ERROR: FileCopy ...'
                    # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
                    LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
                    LResult = False
                #endtry
            case 'Linux':
                # unix
                # LFileNameSource_stat = os.stat (AFileNameSource)
                # Lowner = LFileNameSource_stat [stat.ST_UID]
                # Lgroup = LFileNameSource_stat [stat.ST_GID]
                # os.chown (AFileNameDest, Lowner, Lgroup)
                raise NotImplementedError('FileCopy Linux not implemented...')
            case _:
                s = f'Неизвестная система ...'
                # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
                LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
        #endmatch
    #endif
    return LResult
#endfunction

#-------------------------------------------------------------------------------
# FileMove
#-------------------------------------------------------------------------------
def FileMove (AFileNameSource: str, APathNameDest: str) -> bool:
    """FileMove"""
#beginfunction
    s = f'FileMove: {AFileNameSource:s} -> {APathNameDest:s}'
    # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
    LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)

    if not DirectoryExists(APathNameDest):
        ForceDirectories(APathNameDest)
    #endif
    # LFileNameSource = ExtractFileName (AFileNameSource)
    # LFileNameDest = os.path.join (APathNameDest, LFileNameSource)
    LResult = shutil.move(AFileNameSource, APathNameDest, copy_function=shutil.copy2())
    return LResult
#endfunction

#-------------------------------------------------------------------------------
# CheckFileNameMask
#-------------------------------------------------------------------------------
def CheckFileNameMask (AFileName: str, AMask: str) -> bool:
    """CheckFileNameMask"""
#beginfunction
    if AMask != '':
        LFileName = AFileName
        # LMask = '^[a-zA-Z0-9]+.py$'         # *.py - только латинские буквы и цифры
        # LMask = '^.*..*$'                   # *.* - все символы
        # LMask = '^.*.py$'                   # *.py - все символы
        # LMask = '^[\\S ]*.py$'              # *.py - все символы включая пробелы
        # LMask = '^[a-zA-Z0-9]*.py$'         # *.py - только латинские буквы и цифры
        LMask = AMask
        # print (LMask, LFileName)
        #-------------------------------------------------------------------------------
        # regex = re.compile (LMask)
        # Lresult = regex.match(LFileName)
        #-------------------------------------------------------------------------------
        # эквивалентно
        #-------------------------------------------------------------------------------
        try:
            Lresult = re.match (LMask, LFileName)
            # Lresult = re.search (LMask, LFileName)
        except Exception as e:
            Lresult = False
        #endtry
        #-------------------------------------------------------------------------------
    else:
        Lresult = False
    #endif
    return Lresult
#endfunction

#-------------------------------------------------------------------------------
# CreateTextFile
#-------------------------------------------------------------------------------
def CreateTextFile(AFileName: str, AText: str, AEncoding: str):
    """CreateTextFile"""
#beginfunction
    s = f'CreateTextFile: {AFileName:s} ...'
    # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
    LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)

    LEncoding = AEncoding
    if AEncoding == '':
        # LEncoding = LUStrDecode.cCP1251
        LEncoding = cDefaultEncoding
    #endif
    if len(AText) > 0:
        LHandle = open (AFileName, 'w', encoding = LEncoding)
        LHandle.write (AText + '\n')
        LHandle.flush ()
        LHandle.close ()
    else:
        FileDelete (AFileName)
        LHandle = open (AFileName, 'w', encoding = LEncoding)
        LHandle.flush ()
        LHandle.close ()
   #endif
#endfunction

#--------------------------------------------------------------------------------
# WriteStrToFile
#--------------------------------------------------------------------------------
def WriteStrToFile (AFileName: str, AStr: str, AEncoding: str = ''):
    """WriteStrToFile"""
#beginfunction
    s = f'WriteStrToFile: {AFileName:s} ...'
    # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
    # LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)

    # Откроет для добавления нового содержимого.
    # Создаст новый файл для чтения записи, если не найдет с указанным именем.
    LEncoding = GetFileEncoding (AFileName)
    if AEncoding == '':
        LEncoding = LUStrDecode.cCP1251
        LEncoding = cDefaultEncoding
    else:
        LEncoding = AEncoding
    #endif

    if len(AStr) >= 0:
        LHandle = open (AFileName, 'a+', encoding = LEncoding)
        LHandle.write (AStr + '\n')
        LHandle.flush ()
        LHandle.close ()
    #endif
#endfunction

#-------------------------------------------------------------------------------
# OpenTextFile
#-------------------------------------------------------------------------------
def OpenTextFile(AFileName: str, AEncoding: str) -> int:
    """OpenTextFile"""
#beginfunction
    s = f'OpenTextFile: {AFileName:s} ...'
    # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
    LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)

    LEncoding = AEncoding
    if AEncoding == '':
        # LEncoding = LUStrDecode.cCP1251
        LEncoding = cDefaultEncoding
    #endif
    LHandle = open (AFileName, 'a+', encoding = LEncoding)
    return LHandle
#endfunction

#-------------------------------------------------------------------------------
# WriteTextFile
#-------------------------------------------------------------------------------
def WriteTextFile(AHandle, AStr: str):
    """WriteTextFile"""
#beginfunction
    AHandle.write (AStr+'\n')
    AHandle.flush ()
#endfunction

#-------------------------------------------------------------------------------
# CloseTextFile
#-------------------------------------------------------------------------------
def CloseTextFile (AHandle):
    """CloseTextFile"""
#beginfunction
    s = f'CloseTextFile ...'
    # LULog.LoggerTOOLS_AddLevel (logging.DEBUG, s)
    LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)

    AHandle.flush ()
    AHandle.close ()
#endfunction

#--------------------------------------------------------------------------------
# SearchFile
#--------------------------------------------------------------------------------
def __SearchFile (ADir: str, AFileName: str, AMask: str, AExt: str, ASubDir: bool) -> []:
    """SearchFile"""
#beginfunction
    LList = []
    if ADir == '':
        # искать в текущем каталоге
        LDir = LUos.GetCurrentDir ()
    else:
        # искать в каталоге ADir
        LDir = ExpandFileName (ADir)
    #endif
    Lhere = pathlib.Path (LDir)
    # print('LDir:', LDir)
    LFileName = ExtractFileName (AFileName)
    # print('LFileName:', LFileName)
    # print('AMask:', AMask)
    # print('AExt:', AExt)

    if ASubDir:
        # Searching for LDir Recursively in Python
        LStr = '**/*'
    else:
        # Searching for LDir
        LStr = '*'
    #endif
    # Finding a Single File Recursively
    # LStr = here.glob ("**/something.txt")

    Lfiles = Lhere.glob (LStr)
    for item in Lfiles:
        if item.name == LFileName:
            # print (item)
            LList.append (item)
        else:
            if CheckFileNameMask (item.name, AMask):
                # print (item)
                LList.append (item)
            else:
                if CheckFileExt (item, AExt):
                    # print (item)
                    LList.append (item)
                #endif
            #endif
        #endif
    #endfor
    return LList
#endfunction

#--------------------------------------------------------------------------------
# SearchFileDirs
#--------------------------------------------------------------------------------
def SearchFileDirs (ADirs: [], AFileName: str, AMask: str, AExt: str, ASubDir: bool) -> []:
    """SearchFileDirs"""
#beginfunction
    LListDirs = []
    for LDir in ADirs:
        # print('LDir:', LDir)
        LList = __SearchFile (LDir, AFileName, AMask, AExt, ASubDir)
        if len(LList) > 0:
            LListDirs += LList
        #endif
    #endfor
    return LListDirs
#endfunction

#--------------------------------------------------------------------------------
# GetWindowsPath
#--------------------------------------------------------------------------------
def GetWindowsPath (APath: str) -> str:
    """GetWindowsPat"""
#beginfunction
    LResult = pathlib.WindowsPath (APath)
    return LResult
#endfunction

#--------------------------------------------------------------------------------
# GetPureWindowsPath
#--------------------------------------------------------------------------------
def GetPureWindowsPath (APath: str) -> str:
    """GetPureWindowsPath"""
#beginfunction
    LResult = pathlib.PureWindowsPath (APath)
    return LResult
#endfunction

def sanitize_filename (filename, replacement = '_', platform = None):
    """
    Очищает строку, оставляя только допустимые символы для имени файла.

    :param filename: исходное имя файла
    :param replacement: символ для замены недопустимых символов
    :param platform: целевая платформа ('windows', 'linux', 'darwin' и т.д.), по умолчанию используется текущая ОС
    :return: корректное имя файла
    """
    if not platform:
        platform = os.name

    # Обрезаем до разумной длины (обычно максимум 255 байт)
    max_length = 255

    # Удаление начальных и конечные пробелы
    filename = filename.strip ()

    # Замена недопустимых символов
    if platform == 'nt':  # Windows
        invalid_chars = r'[<>:"/\\|?*\x00-\x1F]'
    else:  # Linux/macOS
        invalid_chars = r'[/\x00]'

    filename = re.sub (invalid_chars, replacement, filename)

    # Удаление лишних точек и подряд идущих заменителей
    filename = re.sub (r'(\.' + re.escape (replacement) + r')+', '.', filename)
    filename = re.sub (re.escape (replacement) + r'{2,}', replacement, filename)

    # Если имя стало пустым — вернуть дефолт
    if not filename:
        filename = f"unnamed_file{replacement}"

    return filename [:max_length]

#-------------------------------------------------------------------------------
# main
#-------------------------------------------------------------------------------
def main ():
#beginfunction
    print('main LUFile.py ...')
#endfunction

#------------------------------------------
# module
#------------------------------------------
#beginmodule
if __name__ == "__main__":
    main()
#endif

#endmodule



"""LUTelegram.py"""
# -*- coding: UTF-8 -*-
__annotations__ = """
 =======================================================
 Copyright (c) 2025
 Author:
     Lisitsin Y.R.
 Project:
     lyrpy
     
 Module:
     LUTelegram.py

 =======================================================
"""

from fontTools.misc.cython import returns
#------------------------------------------
# БИБЛИОТЕКИ python
#------------------------------------------

#------------------------------------------
# БИБЛИОТЕКИ сторонние
#------------------------------------------

#------------------------------------------
# БИБЛИОТЕКА telethon
#------------------------------------------
import telethon.sync
import telethon.tl.types

from telethon.sync import TelegramClient
from telethon.tl.types import PeerUser, PeerChat, PeerChannel
import re

# from telethon.tl.functions.messages import GetDialogsRequest
# from telethon.tl.types import InputPeerEmpty
# from telethon.tl.types import User
# # класс, позволяющий нам подключаться к клиенту мессенджера и работать с ним;
# from telethon.sync import TelegramClient
# # PeerChannel - специальный тип, определяющий объекты типа «канал/чат»,
# # с помощью которого можно обратиться к нужному каналу для парсинга сообщений.
# from telethon.tl.types import Channel, PeerChannel, PeerChat, PeerUser, Message, User, MessageMediaPhoto, MessageMediaDocument
# # конструктор для работы с InputPeer, который передаётся в качестве аргумента в GetDialogsRequest;
# from telethon.tl.types import InputMessagesFilterPhotos
# # функция, позволяющая работать с сообщениями в чате;
# # метод, позволяющий получить сообщения пользователей из чата и работать с ним;
# from telethon.tl.functions.messages import GetHistoryRequest
# from telethon import TelegramClient, events, sync
# import telethon.errors
# import telethon.client.messages as messages

#------------------------------------------
# БИБЛИОТЕКА pyrogram
#------------------------------------------
import pyrogram

#------------------------------------------
# БИБЛИОТЕКИ LU
#------------------------------------------

LIB_name = ''

# ------------------------------------------
# Авторизация в Telegram
# get_telethon_client (session_name, api_id, api_hash, phone, password) -> telethon.sync.TelegramClient:
# ------------------------------------------
def get_telethon_client (session_name, api_id, api_hash, phone, password) -> telethon.sync.TelegramClient:
    """get_telethon_client"""
# beginfunction
    result = telethon.sync.TelegramClient(session_name, api_id, api_hash, system_version="4.16.30-vxNAME ")
    #   Вместо NAME используйте любое сочетание букв на английском КАПСОМ Пример: vxXYI, vxABC, vxMYNAME
    #   # (в папке с кодом нет файлика .session, клиент сам его создаст (в нашем случае 'my_session')
    #   # и будет с ним работать. Поэтому просто вставляем эти параметры в инициализацию и кайфуем:finger_up: )

    # Tclient = TelegramClient (Gsession_name, Gapi_id, Gapi_hash,
    #                           #         device_model = "iPhone 13 Pro Max",
    #                           #         app_version = "8.4",
    #                           #         lang_code = "en",
    #                           #         system_lang_code = "en-US")
    #                           system_version='4.16.30-vxABC')
    # Tclient.start (phone=Gphone, password=Gpassword)
    result.start (phone=phone, password=password)
    result.connect ()
    # print (f'{LIB_name}_user_authorized={result.is_user_authorized()}')
    return result
# endfunction

# ------------------------------------------
# Получить the current User who is logged
# get_telethon_me (client:telethon.sync.TelegramClient) -> telethon.tl.types.User:
# ------------------------------------------
def get_telethon_me (client:telethon.sync.TelegramClient) -> telethon.tl.types.User:
    """get_telethon_me"""
# beginfunction
    result:User = client.get_me ()

    # print (f'{LIB_name}_username={result.username}')
    # print (f'{LIB_name}_phone={result.phone}')

    # print (f'{LIB_name}_stringify={result.stringify()}')
    return result
# endfunction

# ------------------------------------------
# Получить channel
# get_telethon_channel (client:telethon.sync.TelegramClient, channel_name_id) -> telethon.tl.types.Channel:
# ------------------------------------------
def get_telethon_channel (client:telethon.sync.TelegramClient, channel_name_id) -> telethon.tl.types.Channel:
    """get_telethon_channel"""
# beginfunction
    result = client.get_entity (channel_name_id)
    # print (result)
    # print(f'{LIB_name}_Channel.title={result.title}')
    # print(f'{LIB_name}_Channel.id={result.id}')
    # print(f'{LIB_name}_Channel.username={result.id}')
    return result
# endfunction

# ------------------------------------------
# Получить message
# get_telethon_message (client:telethon.sync.TelegramClient, channel:telethon.tl.types.Channel, message_id) -> telethon.tl.types.Message:
# ------------------------------------------
def get_telethon_message (client:telethon.sync.TelegramClient, channel:telethon.tl.types.Channel, message_id) -> telethon.tl.types.Message:
    """get_telethon_message"""
# beginfunction
    try:
        result = client.get_messages (channel.id, ids=message_id)
        # print(f'{LIB_name}_Message={result}')
    except:
        result = None

    return result
# endfunction

#------------------------------------------
# Получить мои группы
# get_telethon_mygroups (client:TelegramClient):
#------------------------------------------
def get_telethon_mygroups (client:telethon.sync.TelegramClient):
    """get_telethon_mygroups"""
#beginfunction
    # Получение списка диалогов
    dialogs = client.get_dialogs()
    # Отображение списка групп
    for dialog in dialogs:
        if dialog.is_group:
            print(f"{LIB_name}_Группа: {dialog.title} (ID: {dialog.id})")
#endfunction

#------------------------------------------
# Получить список groups
#------------------------------------------
def get_telethon_groups (client:telethon.sync.TelegramClient) -> list:
    """get_telethon_groups"""
#beginfunction
    chats = []
    last_date = None
    size_chats = 200
    groups = []

    # -----------------------------------------------
    # Напишем запрос для получения списка групп
    # -----------------------------------------------
    # offset_date и offset_peer мы передаём с пустыми значениями.
    # Обычно они используются для фильтрации полученных данных, но здесь мы
    # хотим получить весь список. Лимит по количеству элементов в ответе задаём 200,
    # передавая в параметр limit переменную size_chats.
    result = client (telethon.tl.functions.messages.GetDialogsRequest (
        offset_date=last_date,
        offset_id=0,
        offset_peer=telethon.tl.types.InputPeerEmpty (),
        limit=size_chats,
        hash=0
    ))
    chats.extend (result.chats)

    # -----------------------------------------------
    #
    # -----------------------------------------------
    for chat in chats:
        try:
            # print (int(chat.megagroup), chat.title)

            # Megagroup в библиотеке Telethon — это свойство объекта Channel,
            # которое указывает, является ли он мегагруппой (большой группой).
            # Мегагруппы — это каналы, для которых значение свойства channel.megagroup равно True.
            # В официальных приложениях Telegram такие группы называются «супергруппами»

            if chat.megagroup == True:
                groups.append (chat)
        except:
            continue
    return groups

#endfunction

#------------------------------------------
# Получить список groups
#------------------------------------------
def get_telethon_groups (client:telethon.sync.TelegramClient) -> list:
    """get_telethon_groups"""
#beginfunction
    chats = []
    last_date = None
    size_chats = 200
    groups = []

    # -----------------------------------------------
    # Напишем запрос для получения списка групп
    # -----------------------------------------------
    # offset_date и offset_peer мы передаём с пустыми значениями.
    # Обычно они используются для фильтрации полученных данных, но здесь мы
    # хотим получить весь список. Лимит по количеству элементов в ответе задаём 200,
    # передавая в параметр limit переменную size_chats.
    result = client (telethon.tl.functions.messages.GetDialogsRequest (
        offset_date=last_date,
        offset_id=0,
        offset_peer=telethon.tl.types.InputPeerEmpty (),
        limit=size_chats,
        hash=0
    ))
    chats.extend (result.chats)
    # -----------------------------------------------
    #
    # -----------------------------------------------
    for chat in chats:
        try:
            # print (int(chat.megagroup), chat.title)
            # Megagroup в библиотеке Telethon — это свойство объекта Channel,
            # которое указывает, является ли он мегагруппой (большой группой).
            # Мегагруппы — это каналы, для которых значение свойства channel.megagroup равно True.
            # В официальных приложениях Telegram такие группы называются «супергруппами»
            if chat.megagroup == True:
                groups.append (chat)
            else:
                # groups.append (chat)
                pass
        except:
            continue
    return groups
#endfunction

#------------------------------------------
# Получить список users group
#------------------------------------------
def get_telethon_users_group (client:telethon.sync.TelegramClient, group) -> list:
    """get_telethon_users_group"""
#beginfunction
    #-----------------------------------------------
    # Узнаём пользователей...
    #-----------------------------------------------
    all_participants = []
    try:
        all_participants = client.get_participants (group, limit=100000)
        # print (group)
        # for user in all_participants:
        #     if user.username:
        #         username = user.username
        #     else:
        #         username = ""
        #     if user.first_name:
        #         first_name = user.first_name
        #     else:
        #         first_name = ""
        #     if user.last_name:
        #         last_name = user.last_name
        #     else:
        #         last_name = ""
        #     name = (first_name + ' ' + last_name).strip ()
        #     print(f'{username=}, {name=}, {target_group.title=}')
    except:
        pass
    return all_participants
#endfunction

#------------------------------------------
# Получить список CHATs
#------------------------------------------
def get_telethon_CHATs (client:telethon.sync.TelegramClient):
    """get_telethon_CHATs"""
#beginfunction
    # понять, канал или чат, можно проверяя у диалога параметр "dialog.entity.megagroup
    for dialog in client.iter_dialogs ():
        if 'Channel' in str (type (dialog.entity)):  # откидываем юзеров
            if dialog.entity.megagroup:
                print (dialog.entity.id, '//', dialog.entity.username,
                       ' - ', dialog.name, ' is CHAT')
#endfunction

#------------------------------------------
# Получить список CHANNELs
#------------------------------------------
def get_telethon_CHANNELs (client:telethon.sync.TelegramClient):
    """get_telethon_CHANNELs"""
#beginfunction
    # понять, канал или чат, можно проверяя у диалога параметр "dialog.entity.megagroup
    for dialog in client.iter_dialogs ():
        if 'Channel' in str (type (dialog.entity)):  # откидываем юзеров
            if not dialog.entity.megagroup:
                print (dialog.entity.id, '//', dialog.entity.username,
                       ' - ', dialog.name, ' is CHANNEL')
#endfunction

# ----------------------------------------------
# Получаем последние 10 сообщений из указанного чата
# ----------------------------------------------
def get_telethon_chat (client:telethon.sync.TelegramClient, chat_id):
    """get_telethon_chat"""
#beginfunction
    # ID чата/канала/пользователя, откуда читать сообщения
    # chat_id = '@GardeZ66'  # или ID (число), или юзернейм (например, '@telegram')
    for message in client.iter_messages (chat_id, limit=10):
        print (f"{LIB_name}_message.sender_id:{message.sender_id}: {message.text}")
#endfunction



# ----------------------------------------------
# Функция для парсинга ссылки
# ----------------------------------------------
def parse_message_link (link):
    """parse_message_link"""
#beginfunction
    pattern = r'https?://t\.me/([a-zA-Z0-9\_]+|c/(\d+))/(\d+)/(\d+)'
    match = re.match (pattern, link)
    if not match:
        raise ValueError ("Invalid link")

    # print(f'{match.group (1)=}')
    # print(f'{match.group (2)=}')
    # print(f'{match.group (3)=}')
    # print(f'{match.group (4)=}')

    if match.group (1).startswith ('c'):
        channel_id = int (match.group (3))
        msg_id = int (match.group (3))
        return {'channel_id': channel_id, 'msg_id': msg_id}
    else:
        username = match.group (1)
        msg_id = int (match.group (3))
        return {'username': username, 'msg_id': msg_id}
#endfunction

# ----------------------------------------------
# async def get_channel_name (link, TelegramClient):
# ----------------------------------------------
def get_channel_name (link, session_name, api_id, api_hash, phone):
    """get_channel_name"""
#beginfunction
    with TelegramClient (session_name, api_id, api_hash) as client:
        client.start (phone)
        # print (f'{link=}')
        parsed = parse_message_link (link)
        print (f'{parsed=}')

        if 'username' in parsed:
            entity = client.get_entity (parsed ['username'])
            # entity = client.get_entity (parsed ['msg_id'])
        elif 'channel_id' in parsed:
            entity = client.get_entity (PeerChannel (parsed ['channel_id']))
        else:
            raise ValueError ("Could not resolve entity")
        print (f"Название канала: {entity.title}")

    # client.disconnect ()

    return entity.title
#endfunction








# ------------------------------------------
# Авторизация в Telegram [pyrogram]
# ------------------------------------------
def get_pyrogram_client (api_id, api_hash, login, phone) -> pyrogram.Client:
    """get_pyrogram_client"""
# beginfunction
    result = pyrogram.Client (login, api_id=api_id, api_hash=api_hash, phone_number=phone)
    result.start ()
    # result.connect ()
    # print (f'{LIB_name}_is_connected={result.is_connected}')

    # print(Tclient.export_session_string())
    # print(result.workdir)

    # result.run ()
    return result
# endfunction

# ------------------------------------------
# Метод client.get_me() в библиотеке Pyrogram возвращает объект pyrogram.User
# с информацией о текущем зарегистрированном пользователе или боте.
# ------------------------------------------
def get_pyrogram_me (client:pyrogram.Client) -> pyrogram.types.User:
    """get_pyrogram_me"""
# beginfunction
    result = client.get_me()
    # print (f'{LIB_name}_username={result.username}')
    # print (f'{LIB_name}_phone_number={result.phone_number}')
    # print (f'pyrogram:stringify={result.stringify()}')
    return result
# endfunction

#---------------------------------------------------------
# main
#---------------------------------------------------------
def main ():
#beginfunction
    pass
#endfunction

#---------------------------------------------------------
#
#---------------------------------------------------------
#beginmodule
if __name__ == "__main__":
    main()
#endif

#endmodule
